# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


__all__ = ['ModelAdapter']

#:
#: prestans.ext.data.adapters.datastore Google DataStore specific 
#: implementation of Data Adapter
#:

import inspect

import prestans.types
import prestans.parser

from prestans.ext.data import adapters

#:
#: Helper methods
#:
def adapt_persistent_instance(persistent_object, target_rest_class=None, attribute_filter=None):
    """
    Adapts a single persistent instance to a REST model; at present this is a
    common method for all persistent backends.

    This might be moved to backend specific packages if the need arrises

    Refer to: https://groups.google.com/forum/#!topic/prestans-discuss/dO1yx8f60as
    for discussion on this feature
    """

    #: Try and get the adapter and the REST class for the persistent object 
    if target_rest_class is None:
        adapter_instance = adapters.registry.get_adapter_for_persistent_model(collection[0])
    else:
        if inspect.isclass(target_rest_class):
            target_rest_class = target_rest_class()
        
        adapter_instance = adapters.registry.get_adapter_for_rest_model(target_rest_class)

    #: Would raise an exception if the attribute_filter differs from the target_rest_class
    if attribute_filter is not None and isinstance(attribute_filter, prestans.parser.AttributeFilter):
        prestans.parser.AttributeFilter.from_model(target_rest_class).conforms_to_template_filter(attribute_filter)

    return adapter_instance.adapt_persistent_to_rest(persistent_object, attribute_filter)

#:
#: base implementation that translates record sets into prestans arrays
#:
def adapt_persistent_collection(persistent_collection, target_rest_class=None, attribute_filter=None):
        
    #: Ensure that colleciton is iterable and has atleast one element
    persistent_collection_length = 0
        
    #: Attempt to reliably detect the length of the persistent_collection
    if persistent_collection and isinstance(persistent_collection, (list, tuple)):
        persistent_collection_length = len(persistent_collection)
    elif persistent_collection:
        persistent_collection_length = persistent_collection.count()

    #: If the persistent_collection is empty then return a blank array 
    if persistent_collection_length == 0:
        return prestans.types.Array(element_template=target_rest_class())
        
    #: Try and get the adapter and the REST class for the persistent object 
    if target_rest_class is None:
        adapter_instance = adapters.registry.get_adapter_for_persistent_model(persistent_collection[0])
    else:
        if inspect.isclass(target_rest_class):
            target_rest_class = target_rest_class()
        
        adapter_instance = adapters.registry.get_adapter_for_rest_model(target_rest_class)
    
    adapted_models = prestans.types.Array(element_template=adapter_instance.rest_model_class())
    
    for persistent_object in persistent_collection:
        adapted_models.append(adapter_instance.adapt_persistent_to_rest(persistent_object))
        
    return adapted_models
 
#:
#: Provide a brige between REST models and Google Datastore objects
#:  
class ModelAdapter(adapters.ModelAdapter):
       
    #: 
    #: adapts a persistent model to a rest model by inspecting
    #:
    #: @todo test all data types provided by datastore
    #:
    def adapt_persistent_to_rest(self, persistent_object, attribute_filter=None):

        rest_model_instance = self.rest_model_class()
        
        for attribute_key in rest_model_instance.get_attribute_keys():
                            
            rest_attr = getattr(self.rest_model_class, attribute_key)
            
            if not hasattr(persistent_object, attribute_key):
            
                #: Don't bother processing if the persistent model doesn't have this attribute
                if issubclass(rest_attr.__class__, prestans.types.Model):
                    #: If the attribute is a Model, then we set it to None otherwise we get a model 
                    #: with default values, which is invalid when constructing responses
                    setattr(rest_model_instance, attribute_key, None)
                
                continue

            #: Attribute not visible don't bother processing
            elif isinstance(attribute_filter, prestans.parser.AttributeFilter) and\
             not attribute_filter.is_attribute_visible(attribute_key):
                continue

            elif isinstance(rest_attr, prestans.types.Array):

                persistent_attr_value = getattr(persistent_object, attribute_key)
                rest_model_array_handle = getattr(rest_model_instance, attribute_key)

                #: Iterator uses the .append method exposed by prestans arrays to validate
                #: and populate the collection in the instance.
                for collection_element in persistent_attr_value:
                 
                    if isinstance(rest_attr.element_template, prestans.types.String):
                        rest_model_array_handle.append(collection_element)
                    elif isinstance(rest_attr.element_template, prestans.types.Integer):
                        rest_model_array_handle.append(collection_element)
                    elif isinstance(rest_attr.element_template, prestans.types.Float):
                        rest_model_array_handle.append(collection_element)
                    elif isinstance(rest_attr.element_template, prestans.types.Boolean):
                        rest_model_array_handle.append(collection_element)
                    else:
                        element_adapter = adapters.registry.get_adapter_for_rest_model(rest_attr._element_template)
                        adapted_rest_model = element_adapter.adapt_persistent_to_rest(collection_element)                    
                        rest_model_array_handle.append(adapted_rest_model)

            elif isinstance(rest_attr, prestans.types.Model):

                try:
                    
                    persistent_attr_value = getattr(persistent_object, attribute_key)
                    
                    if persistent_attr_value is None:
                        adapted_rest_model = None
                    else:
                        model_adapter = adapters.registry.get_adapter_for_rest_model(rest_attr)
                        adapted_rest_model = model_adapter.adapt_persistent_to_rest(persistent_attr_value)
                    
                    setattr(rest_model_instance, attribute_key, adapted_rest_model)
                    
                except TypeError, exp:
                    raise TypeError('Attribute %s, %s' % (attribute_key, str(exp)))
                except prestans.exception.DataValidationException, exp:
                    raise prestans.exception.InconsistentPersistentDataError(attribute_key, str(exp))

            else:
            
                # Default operation is to copy the value and set it, validate will ensure its an acceptable value
                try:
                    persistent_attr_value = getattr(persistent_object, attribute_key)
                    setattr(rest_model_instance, attribute_key, persistent_attr_value)
                except TypeError, exp:
                    raise TypeError('Attribute %s, %s' % (attribute_key, str(exp)))
                except prestans.exception.DataValidationException, exp:
                    raise prestans.exception.InconsistentPersistentDataError(attribute_key, str(exp))
            
        return rest_model_instance



# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from functools import wraps

import prestans.exception

class Base(object):

    def __init__(self):
        self._debug = False

    @property
    def debug(self):
        return self._debug

    @debug.setter
    def debug(self, value):
        self._debug = value

    def current_user_has_role(self, role_name):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)
    
    def is_authenticated_user(self, handler_reference):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)
        
    def is_authorized_user(self, handler_reference):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)

    def get_current_user(self):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)


def login_required(http_method_handler):
    """
    provides a decorator for RESTRequestHandler methods to check for authenticated users

    RESTRequestHandler subclass must have a auth_context instance, refer to prestans.auth
    for the parent class definition.

    If decorator is used and no auth_context is provided the client will be denied access

    Handler will return a 401 Unauthorized if the user is not logged in, the service does not redirect
    to login handler page, this is the client's responsibility

    auth_context_handler instance provides a message called get_current_user, use this to obtain a
    reference to an authenticated user profile.

    If all goes well, the original handler definition is executed.

    """
    
    @wraps(http_method_handler)
    def secure_http_method_handler(self, *args):
            
        if not self.__provider_config__.authentication:
            _message = "Service available to authenticated users only, no auth context provider set in handler"
            raise prestans.exception.AuthenticationError(_message)
            
        if not self.__provider_config__.authentication.is_authenticated_user():
            raise prestans.exception.AuthenticationError()

        http_method_handler(self, *args)
        
    return secure_http_method_handler
    
def role_required(role_name=None):
    """
    Authenticates a HTTP method handler based on a provided role

    With a little help from Peter Cole's Blog
    http://mrcoles.com/blog/3-decorator-examples-and-awesome-python/

    """
    
    def _role_required(http_method_handler):

        def secure_http_method_handler(self, *args):
    
            # Role name must be provided
            if role_name is None:
                raise prestans.exception.AuthorizationError("None")
        
            # Authentication context must be set
            if not self.__provider_config__.authentication:
                _message = "Service available to authenticated users only, no auth context provider set in handler"
                raise prestans.exception.AuthenticationError(_message)
            
            # Check for the role by calling current_user_has_role
            if not self.__provider_config__.authentication.current_user_has_role(role_name):
                raise prestans.exception.AuthorizationError(role_name)

            http_method_handler(self, *args)
        
        return wraps(http_method_handler)(secure_http_method_handler)
        
    return _role_required

def access_required(config=None):
    """
    Authenticates a HTTP method handler based on a custom set of arguments
    """
    
    def _access_required(http_method_handler):

        def secure_http_method_handler(self, *args):

            # Authentication context must be set
            if not self.__provider_config__.authentication:
                _message = "Service available to authenticated users only, no auth context provider set in handler"
                raise prestans.exception.AuthenticationError(_message)

            # Check for access by calling is_authorized_user
            if not self.__provider_config__.authentication.is_authorized_user(config):
                _message = "Service available to authorized users only"
                raise prestans.exception.AuthorizationError(_message)

            http_method_handler(self, *args)
        
        return wraps(http_method_handler)(secure_http_method_handler)
        
    return _access_required


# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__all__ = ['AppEngineAuthContextProvider', 'AppEngineCacheProvider']

## @package prestans.appengine Providers Google AppEngine specific implementation of providers
#
# appengine is not imported as part of "all", this has to be explicity imported by the user.
#

from prestans.provider import auth
from prestans.provider import cache
from prestans.provider import throttle

from google.appengine.api import oauth
from google.appengine.api import users
from google.appengine.api import memcache

import os

_IS_DEVELOPMENT_SERVER = os.environ.get('SERVER_SOFTWARE', '').startswith('Development')

#:
#: Provides Authentication Context for Google's AppEngine Environment
#:
#:
#: auth.AuthContextProvider implementation for Google AppEngine, uses google.appengine.api.users
#: to determine logged in users and return a reference to the current user.
#:
#: Refer to decorators in handlers package for more information.
#:
class AppEngineAuthContextProvider(auth.Base):
    
    #:
    #: Overriden is_authenticated_user for Google AppEngine
    #: Uses google.appengine.api.users method get_current_user to check if a user is logged in 
    #:
    def is_authenticated_user(self):
        return users.get_current_user() is not None
      
    #:  
    #: Overriden get_current_user for Google AppEngine
    #: Checks for oauth capable request first, if this fails fall back to standard users API
    #:
    def get_current_user(self):

        if _IS_DEVELOPMENT_SERVER:
            return users.get_current_user()
        else:
            try:
                user = oauth.get_current_user()
            except oauth.OAuthRequestError, exp:
                user = users.get_current_user()
            return user
  
#:      
#: Provides a wrapper on AppEngine's memcache implementation 
#:
class AppEngineMemcacheProvider(cache.Base):
    
    def __init__(self, key_provider=None):
        self.key_provider = key_provider
    
    def get(self, key):
        memcache.get(key)       
    
    def set(self, key, value, expiry=60, namespace=None):
        memcache.set(key, value, expiry)
        
    def add(self, key, value, expiry=60, namespace=None):
        memcache.add(key, value, expiry, namespace)
        
    def delete(self, key, seconds=0, namespace=None):
        memcache.delete(key, seconds, namespace)
        
    def flush_all(self):
        memcache.flush_all()




# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__all__ = ['Config', 'auth', 'cache', 'throttle']

class Config(object):
	
	def __init__(self, authentication=None, throttle=None, cache=None):
		self._authentication = authentication
		self._throttle = throttle
		self._cache = cache

	def blueprint(self):
		provider_blueprint = dict()

		return provider_blueprint

	@property
	def authentication(self):
		return self._authentication

	@authentication.setter
	def authentication(self, value):
		self._authentication = value

	@property
	def throttle(self):
		return self._throttle

	@throttle.setter
	def throttle(self, value):
		self._throttle = value

	@property
	def cache(self):
		return self._cache

	@cache.setter
	def cache(self, value):
		self._cache = value

	


# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import re
import os
import copy
import string
import inspect
import base64

from datetime import datetime
from datetime import date
from datetime import time

import prestans.parser
import prestans.exception

#:
#: Base type classes
#: 

class DataType(object):
    
    def validate(self, value):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)

class DataStructure(DataType):
    """
    Wrappers on Python types generally represented as structures e.g DateTime

    as_serializable methods signature for %DataStructure is different to that of DataCollection
    it requires a value to be passed in, this is because the python type of structures is 
    difference to what gets serialized.

    E.g DateTime serializes itself as a ISO string
    """
    
    def as_serializable(self, value):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)

class DataCollection(DataType):

    def validate(self, value, attribute_filter=None):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)

    def as_serializable(self, attribute_filter=None):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)

    def get_attribute_filter(self):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)

#:
#: Basic Types
#:

class String(DataType):
    
    def __init__(self, default=None, min_length=None, max_length=None, 
        required=True, format=None, choices=None, utf_encoding='utf-8', description=None):

        if min_length and max_length and min_length > max_length:
            pass

        if required and min_length and min_length < 1:
            pass

        self._default = default
        self._min_length = min_length
        self._max_length = max_length
        self._required = required
        self._format = format
        self._choices = choices
        self._utf_encoding = utf_encoding
        self._description = description

    @property
    def max_length(self):
        return self._max_length

    @property
    def min_length(self):
        return self._min_length

    @property
    def default(self):
        return self._default

    @property
    def choices(self):
        return self._choices

    @property
    def format(self):
        return self._format

    @property
    def utf_encoding(self):
        return self._utf_encoding

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'string'

        constraints = dict()
        constraints['default'] = self._default
        constraints['min_length'] = self._min_length
        constraints['max_length'] = self._max_length
        constraints['required'] = self._required
        constraints['format'] = self._format
        constraints['choices'] = self._choices
        constraints['utf_encoding'] = self._utf_encoding
        constraints['description'] = self._description

        blueprint['constraints'] = constraints

        return blueprint

    def validate(self, value):

        _validated_value = None
        
        if self._required and self._default is None and value is None:
            raise prestans.exception.RequiredAttributeError()
        elif self._required and value is None:
            value = self._default
        elif not self._required and self._default is None and value is None:
            return _validated_value
        elif not self._required and value is None:
            value = self._default
        
        try:
            if isinstance(value, unicode):
                _validated_value = u''.join(value).encode(self._utf_encoding).strip()
            else:
                _validated_value = str(value)
        except Exception, exp:
            raise prestans.exception.ParseFailedError("unicode or string encoding failed, %" % exp)
        
        if not self._required and len(_validated_value) == 0:
            return _validated_value
        
        if _validated_value is not None and self._min_length and len(_validated_value) < self._min_length:
            raise prestans.exception.UnacceptableLengthError(value, self._min_length, self._max_length)
        if _validated_value is not None and self._max_length and len(_validated_value) > self._max_length:
            raise prestans.exception.UnacceptableLengthError(value, self._min_length, self._max_length)
            
        if self._choices is not None and not _validated_value in self._choices:
            raise prestans.exception.InvalidChoiceError(value, self._choices)
            
        if self._format is not None and re.search(self._format, _validated_value) is None:
            raise prestans.exception.InvalidFormatError(_validated_value)
        
        return _validated_value

class Integer(DataType):

    def __init__(self, default=None, minimum=None, maximum=None, 
        required=True, choices=None, description=None):

        if minimum and maximum and minimum > maximum:
            pass

        self._default = default
        self._minimum = minimum
        self._maximum = maximum
        self._required = required
        self._choices = choices
        self._description = description

    @property
    def minimum(self):
        return self._minimum

    @property
    def maximum(self):
        return self._maximum

    @property
    def default(self):
        return self._default

    @property
    def choices(self):
        return self._choices

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'integer'

        constraints = dict()
        constraints['default'] = self._default
        constraints['minimum'] = self._minimum
        constraints['maximum'] = self._maximum
        constraints['required'] = self._required
        constraints['choices'] = self._choices
        constraints['description'] = self._description

        blueprint['constraints'] = constraints

        return blueprint

    def validate(self, value):

        _validated_value = None
        
        if self._required and self._default is None and value is None:
            raise prestans.exception.RequiredAttributeError()
        elif self._required and value is None:
            value = self._default
        elif not self._required and self._default is None and value is None:
            return _validated_value
        elif not self._required and value is None:
            value = self._default
        
        try:
            _validated_value = int(value)
        except Exception, exp:
            raise prestans.exception.ParseFailedError("int encoding failed %" % exp)
        
        if _validated_value and self._minimum is not None and _validated_value < self._minimum:
            raise prestans.exception.LessThanMinimumError(value, self._minimum)
        if _validated_value and self._maximum is not None and _validated_value > self._maximum:
            raise prestans.exception.MoreThanMaximumError(value, self._maximum)
            
        if self._choices is not None and not _validated_value in self._choices:
            raise prestans.exception.InvalidChoiceError(value, self._choices)
        
        return _validated_value

class Float(DataType):

    def __init__(self, default=None, minimum=None, maximum=None, required=True, 
        choices=None, description=None):
        
        if minimum and maximum and minimum > maximum:
            pass
        
        self._default = default
        self._minimum = minimum
        self._maximum = maximum
        self._required = required
        self._choices = choices
        self._description = description

    @property
    def minimum(self):
        return self._minimum

    @property
    def maximum(self):
        return self._maximum

    @property
    def default(self):
        return self._default

    @property
    def choices(self):
        return self._choices

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'float'
        
        constraints = dict()
        constraints['default'] = self._default
        constraints['minimum'] = self._minimum
        constraints['maximum'] = self._maximum
        constraints['required'] = self._required
        constraints['choices'] = self._choices
        constraints['description'] = self._description

        blueprint['constraints'] = constraints
        return blueprint

    def validate(self, value):

        _validated_value = None
        
        if self._required and self._default is None and value is None:
            raise prestans.exception.RequiredAttributeError()
        elif self._required and value is None:
            value = self._default
        elif not self._required and self._default is None and value is None:
            return _validated_value
        elif not self._required and value is None:
            value = self._default
        
        try:
            _validated_value = float(value)
        except Exception, exp:
            raise prestans.exception.ParseFailedError("float encoding failed %" % exp)
        
        if _validated_value and self._minimum is not None and _validated_value < self._minimum:
            raise prestans.exception.LessThanMinimumError(value, self._minimum)
        if _validated_value and self._maximum is not None and _validated_value > self._maximum:
            raise prestans.exception.MoreThanMaximumError(value, self._maximum)
            
        if self._choices is not None and not _validated_value in self._choices:
            raise prestans.exception.InvalidChoiceError(value, self._choices)
        
        return _validated_value

        
class Boolean(DataType):

    def __init__(self, default=None, required=True, description=None):

        self._default = default
        self._required = required
        self._description = description

    @property
    def default(self):
        return self._default

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'boolean'

        constraints = dict()
        constraints['default'] = self._default
        constraints['required'] = self._required
        constraints['description'] = self._description

        blueprint['constraints'] = constraints
        return blueprint

    def validate(self, value):

        _validated_value = None
        
        if self._required and self._default is None and value is None:
            raise prestans.exception.RequiredAttributeError()
        elif self._required and value is None:
            value = self._default
        elif not self._required and self._default is None and value is None:
            return _validated_value
        elif not self._required and value is None:
            value = self._default
        
        try:
            _validated_value = bool(value)
        except Exception, exp:
            raise prestans.exception.ParseFailedError()
        
        return _validated_value


class DataURLFile(DataStructure):
    """
    Accepts a Fileupload as part of the JSON body using FileReader's readAsDataURL

    readAsDataURL, encodes the contents of the file as a DataURLScheme, 
    http://en.wikipedia.org/wiki/Data_URI_scheme

    Example
    http://www.html5rocks.com/en/tutorials/file/dndfiles/

    Meta information about the file upload is upto the implementing application
    """

    @classmethod
    def generate_filename(cls):
        import uuid
        return uuid.uuid4().hex

    def __init__(self, required=True, allowed_mime_types=[], description=None):

        self._required = required
        self._allowed_mime_types = allowed_mime_types
        self._description = description

        if isinstance(allowed_mime_types, str):
            self._allowed_mime_types = [allowed_mime_types]

        self._mime_type = None
        self._file_contents = None

    @property
    def allowed_mime_types(self):
        return self._allowed_mime_types

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'data_url_file'

        constraints = dict()
        constraints['required'] = self._required
        constraints['allowed_mime_types'] = self._allowed_mime_types
        constraints['description'] = self._description

        blueprint['constraints'] = constraints
        return blueprint

    @property
    def mime_type(self):
        return self._mime_type

    @property
    def file_contents(self):
        return self._file_contents

    @property
    def base64_contents(self):
        return base64.b64encode(self._file_contents) 

    def validate(self, value):

        _validated_value = self.__class__()

        if self._required and value is None:
            raise prestans.exception.RequiredAttributeError()

        if self._required is False and value is None:
            return value

        try:
            data_url, delimiter, base64_content = value.partition(',')
            _validated_value._mime_type = data_url.replace(';base64', '').replace('data:', '')
            _validated_value._file_contents = base64.b64decode(base64_content)
        except Exception, exp:
            raise prestans.exception.ParseFailedError("data url file encoding failed %" % exp)

        if self._allowed_mime_types and len(self._allowed_mime_types) > 0 \
        and not _validated_value._mime_type in self._allowed_mime_types:
            raise prestans.exception.InvalidChoiceError(_validated_value._mime_type, self._allowed_mime_types)

        return _validated_value

    def save(self, path):
        """
        Writes file to a particular location

        This won't work for cloud environments like Google's Appengine, use with caution
        ensure to catch exceptions so you can provide informed feedback.

        prestans does not mask File IO exceptions so your handler can respond better.
        """
        
        file_handle = open(path, 'wb')
        file_handle.write(self._file_contents)
        file_handle.close()

    def as_serializable(self, value):
        #: This is passed in a DataURLFile and we construct a String back from it
        return "data:%s;base64,%s" % (value.mime_type, value.base64_contents)

#:
#: DataStructures
#: Default format is the Date Time Format string, defaults to RFC822
#:

class DateTime(DataStructure):

    class CONSTANT:
        NOW = '_PRESTANS_CONSTANT_MODEL_DATETIME_NOW'

    def __init__(self, default=None, required=True, format="%Y-%m-%d %H:%M:%S", timezone=False, utc=False, description=None):

        self._default = default
        self._required = required
        self._timezone = timezone
        self._utc = utc
        self._format = format
        self._description = description

    @property
    def default(self):
        return self._default

    @property
    def format(self):
        return self._format

    @property
    def timezone(self):
        return self._timezone

    @property
    def utc(self):
        return self._utc

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'datetime'

        constraints = dict()
        constraints['default'] = self._default
        constraints['required'] = self._required
        constraints['format'] = self._format
        constraints['timezone'] = self._timezone
        constraints['utc'] = self._utc
        constraints['description'] = self._description

        blueprint['constraints'] = constraints
        return blueprint

    def validate(self, value):

        _validated_value = None
        
        if self._required and self._default is None and value is None:
            raise prestans.exception.RequiredAttributeError()
        elif self._required and value is None:
            if self._default == DateTime.CONSTANT.NOW:
                value = datetime.now()
            else:
                value = self._default
        elif not self._required and self._default is None and value is None:
            return _validated_value
        elif not self._required and value is None:
            if self._default == DateTime.CONSTANT.NOW:
                value = datetime.now()
            else:
                value = self._default
        
        if type(value) == datetime:
            _validated_value = value
        elif type(value) == str or type(value) == unicode:
            try:
                _validated_value = datetime.strptime(value, self._format)
            except ValueError, exp:
                raise prestans.exception.ParseFailedError("date time parsing failed %" % exp)
        else:
            raise prestans.exception.ParseFailedError("cannot parse value of type %s" % value.__class__.__name__)
            
        return _validated_value

    def as_serializable(self, value):

        if not type(value) == datetime:
            raise prestans.exception.InvalidTypeError(value, 'datetime.datetime')
            
        return value.strftime(self._format)

class Date(DataStructure):

    class CONSTANT:
        TODAY = '_PRESTANS_CONSTANT_MODEL_DATE_TODAY'
    
    def __init__(self, default=None, required=True, format="%Y-%m-%d", description=None):

        self._default = default
        self._required = required
        self._format = format
        self._description = description

    @property
    def default(self):
        return self._default

    @property
    def format(self):
        return self._format

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'date'

        constraints = dict()
        constraints['default'] = self._default
        constraints['required'] = self._required
        constraints['format'] = self._format
        constraints['description'] = self._description

        blueprint['constraints'] = constraints
        return blueprint

    def validate(self, value):
        
        _validated_value = None
        
        if self._required and self._default is None and value is None:
            raise prestans.exception.RequiredAttributeError()
        elif self._required and value is None:
            if self._default == Date.CONSTANT.TODAY:
                value = date.today()
            else:
                value = self._default
        elif not self._required and self._default is None and value is None:
            return _validated_value
        elif not self._required and value is None:
            if self._default == Date.CONSTANT.TODAY:
                value = date.today()
            else:
                value = self._default
        
        if type(value) == date:
            _validated_value = value
        elif type(value) == str or type(value) == unicode:
            try:
                _validated_value = datetime.strptime(value, self._format).date()
            except ValueError, exp:
                raise prestans.exception.ParseFailedError("date parsing failed %" % exp)
        else:
            raise prestans.exception.ParseFailedError("cannot parse value of type %s" % value.__class__.__name__)
            
        return _validated_value

    def as_serializable(self, value):

        if not type(value) == date:
            raise prestans.exception.InvalidTypeError(value, 'datetime.date')
            
        return value.strftime(self._format)

class Time(DataStructure):

    class CONSTANT:
        NOW = '_PRESTANS_CONSTANT_MODEL_TIME_NOW'
    
    def __init__(self, default=None, required=True, format="%H:%M:%S", description=None):

        self._default = default
        self._required = required
        self._format = format
        self._description = description

    @property
    def default(self):
        return self._default

    @property
    def format(self):
        return self._format

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'time'

        constraints = dict()
        constraints['default'] = self._default
        constraints['required'] = self._required
        constraints['format'] = self._format
        constraints['description'] = self._description

        blueprint['constraints'] = constraints
        return blueprint

    def validate(self, value):

        _validated_value = None
        
        if self._required and self._default is None and value is None:
            raise prestans.exception.RequiredAttributeError()
        elif self._required and value is None:
            if self._default == Time.CONSTANT.NOW:
                value = time.today()
            else:
                value = self._default
        elif not self._required and self._default is None and value is None:
            return _validated_value
        elif not self._required and value is None:
            if self._default == Time.CONSTANT.NOW:
                value = time.today()
            else:
                value = self._default
        
        if type(value) == time:
            _validated_value = value
        elif type(value) == str or type(value) == unicode:
            try:
                _validated_value = datetime.strptime(value, self._format).time()
            except ValueError, exp:
                raise prestans.exception.ParseFailedError("time parsing failed %" % exp)
        else:
            raise prestans.exception.ParseFailedError("cannot parse value of type %s" % value.__class__.__name__)

        return _validated_value

    def as_serializable(self, value):

        if not type(value) == time:
            raise prestans.exception.InvalidTypeError(value, 'datetime.time')
            
        return value.strftime(self._format)

#:
#: Collections
#:

class Array(DataCollection):
    
    def __init__(self, default=None, required=True, element_template=None, 
        min_length=None, max_length=None, description=None):

        if not isinstance(element_template, DataType):
            raise TypeError("Array element_template must a DataType subclass; %s given" % 
                element_template.__class__.__name__)

        #:
        #: Force required to be True if basic type in  use
        #:
        if isinstance(element_template, DataType)\
         and not isinstance(element_template, DataCollection)\
          and not isinstance(element_template, DataStructure):
            element_template._required = True

        self._default = default
        self._required = required
        self._element_template = element_template
        self._min_length = min_length
        self._max_length = max_length
        self._description = description
        
        self._array_elements = list()

    def __len__(self):
        return len(self._array_elements)
        
    def __iter__(self):
        #:
        #: With a little help from 
        #: http://johnmc.co/llum/the-easiest-way-to-implement-__iter__-for-a-python-object/
        #:
        for element in self._array_elements:
            yield element

    def __getitem__(self, index):
        return self._array_elements[index]

    def __constains__(self, item):
        return item in self._array_elements

    @property
    def max_length(self):
        return self._max_length

    @property
    def min_length(self):
        return self._min_length

    @property
    def default(self):
        return self._default

    @property
    def element_template(self):
        return self._element_template

    @element_template.setter
    def element_template(self, value):
        self._element_template = value

    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'array'

        constraints = dict()
        # constraints['default'] = self._default
        constraints['required'] = self._required
        constraints['min_length'] = self._min_length
        constraints['max_length'] = self._max_length
        constraints['element_template'] = self._element_template.blueprint()
        constraints['description'] = self._description

        blueprint['constraints'] = constraints
        return blueprint

    def remove(self, value):
        self._array_elements.remove(value)
    
    def validate(self, value, attribute_filter=None, minified=False):
        
        if not self._required and not value:
            return None

        _validated_value = self.__class__(element_template=self._element_template, 
                                     min_length=self._min_length, 
                                     max_length=self._max_length)
        
        if not isinstance(value, (list, tuple)):
            raise TypeError(value)
            
        for array_element in value:
    
            if isinstance(self._element_template, DataCollection):
                validated_array_element = self._element_template.validate(array_element, attribute_filter)
            else:
                validated_array_element = self._element_template.validate(array_element)
    
            _validated_value.append(validated_array_element)
    
        if self._min_length is not None and len(_validated_value) < self._min_length:
            raise prestans.exception.LessThanMinimumError(value, self._minimum)

        if self._max_length is not None and len(_validated_value) > self._max_length:
            raise prestans.exception.MoreThanMaximumError(value, self._maximum)

        return _validated_value
    
    def append(self, value):
        
        if isinstance(value, (list, tuple)):

            for element in value:
                self.append(element)
            return

        #if not isinstance(value, DataType):
        #    raise TypeError("value must DataType subclass; %s given" % 
        #        value.__class__.__name__)
        
        if isinstance(self._element_template, String) and \
        isinstance(value, str):
            value = self._element_template.__class__().validate(value)
        elif isinstance(self._element_template, String) and \
        isinstance(value, unicode):
            value = self._element_template.__class__().validate(value)
        elif isinstance(self._element_template, Integer) and \
        isinstance(value, int):
            value = self._element_template.__class__().validate(value)
        elif isinstance(self._element_template, Integer) and \
        isinstance(value, long):
            value = self._element_template.__class__().validate(value)
        elif isinstance(self._element_template, Float) and \
        isinstance(value, float):
            value = self._element_template.__class__().validate(value)
        elif isinstance(self._element_template, Boolean) and \
        isinstance(value, bool):
            value = self._element_template.__class__().validate(value)
        elif not isinstance(value, self._element_template.__class__):
            raise TypeError("prestans array elements must be of type %s; given %s" % (self._element_template.__class__.__name__, value.__class__.__name__))
        
        self._array_elements.append(value)
            
    def as_serializable(self, attribute_filter=None, minified=False):

        _result_array = list()
            
        for array_element in self._array_elements:
        
            if isinstance(array_element, str) or \
            isinstance(array_element, unicode) or \
            isinstance(array_element, float) or \
            isinstance(array_element, int) or \
            isinstance(array_element, bool):

                _result_array.append(array_element)

            elif isinstance(array_element, DataStructure):
                serializable_value = self._element_template.as_serializable(array_element)
                _result_array.append(serializable_value)

            elif isinstance(array_element, DataCollection):
                # Assume that this is a model
                _result_array.append(array_element.as_serializable(attribute_filter, minified))
        
        return _result_array

    def attribute_rewrite_map(self):
        return self._element_template.attribute_rewrite_map()

    def attribute_rewrite_reverse_map(self):
        return self._element_template.attribute_rewrite_reverse_map()
 
    def get_attribute_filter(self, default_value=False):

        attribute_filter = None

        if isinstance(self._element_template, DataCollection):
            attribute_filter = self._element_template.get_attribute_filter(default_value)
        elif isinstance(self._element_template, DataType) or \
            isinstance(self._element_template, DataStructure):
            attribute_filter = default_value

        return attribute_filter

#:
#: Models
#:
        
class Model(DataCollection):

    def __init__(self, required=True, default=None, description=None, **kwargs):
        """
        If you are using the Model constructor to provide Meta data, you can provide it
        a default dictionary to initialise instance to initalise it from

        Can run _create_instance_attributes to copy attribute tempaltes to the instance,
        Model instances do not have any configurable options.

        @param required whether or not this model is required when used as an attribute
        @param default provides a default value for your type, used if one is not provided
        @param kwargs Named parameters of which to instantiate the model with
        """

        self._required = required
        self._default = default
        self._description = description

        self._create_instance_attributes(kwargs)

    def default(self):
        return self._default

    def attribute_count(self):

        count = 0
        model_class_members = inspect.getmembers(self.__class__)
        
        for attribute_name, type_instance in model_class_members:
            
            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue
            
            if isinstance(type_instance, DataType):
                _attribute_keys.append(attribute_name)

        return count
                
    def blueprint(self):

        blueprint = dict()
        blueprint['type'] = 'model'
        blueprint['description'] = inspect.getdoc(self.__class__)

        constraints = dict()
        constraints['required'] = self._required
        constraints['model_template'] = self.__class__.__name__
        constraints['description'] = self._description

        blueprint['constraints'] = constraints

        fields = dict()
        model_class_members = inspect.getmembers(self.__class__)
    
        for attribute_name, type_instance in model_class_members:

            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue

            if not isinstance(type_instance, DataType):
                raise TypeError("%s must be of a DataType subclass" % attribute_name)

            fields[attribute_name] = type_instance.blueprint()

        blueprint['fields'] = fields
        return blueprint

    def __setattr__(self, key, value):
        
        if key[0:1] == "_":
            self.__dict__[key] = value
            return
        
        model_class_members = inspect.getmembers(self.__class__)
        
        validator = None
        for attribute_name,  type_instance in model_class_members:
            if attribute_name == key:
                validator = type_instance
            
        if validator is not None:

            try:
                if validator.__class__ == value.__class__:
                    self.__dict__[key] = value
                else:
                    self.__dict__[key] = validator.validate(value)

            except prestans.exception.DataValidationException, exp:
                    raise prestans.exception.ValidationError(
                    message=str(exp), 
                    attribute_name=key, 
                    value=value, 
                    blueprint=validator.blueprint())
            
            return
            
        raise KeyError("No key named %s; in instance of type %s " % (key, self.__class__.__name__))
    
    def _create_instance_attributes(self, arguments):
        """
        Copies class level attribute templates and makes instance placeholders

        This step is required for direct uses of Model classes. This creates a copy of attribute_names
        ignores methods and private variables. DataCollection types are deep copied to ignore memory
        reference conflicts.

        DataType instances are initialized to None or default value.
        """

        model_class_members = inspect.getmembers(self.__class__)

        for attribute_name, type_instance in model_class_members:

            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue

            if isinstance(type_instance, DataCollection):
                self.__dict__[attribute_name] = copy.deepcopy(type_instance)
                continue
                
            if type_instance is None:
                self.__dict__[attribute_name] = None
                continue
                
            if isinstance(type_instance, DataType):
                
                try:
                    value = None
                    
                    if arguments.has_key(attribute_name):
                        value = arguments[attribute_name]

                    self.__dict__[attribute_name] = type_instance.validate(value)

                except prestans.exception.DataValidationException, exp:
                    self.__dict__[attribute_name] = None

    def get_attribute_keys(self):
        """
        Returns a list of managed attributes for the Model class

        Implemented for use with data adapters, can be used to quickly make a list of the 
        attribute names in a prestans model
        """

        _attribute_keys = list()
        
        model_class_members = inspect.getmembers(self.__class__)
        
        for attribute_name, type_instance in model_class_members:
            
            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue
            
            if isinstance(type_instance, DataType):
                _attribute_keys.append(attribute_name)
            
        return _attribute_keys

    def get_attribute_filter(self, default_value=False):

        attribute_filter = prestans.parser.AttributeFilter()

        _model_class_members = inspect.getmembers(self.__class__)

        for attribute_name, type_instance in _model_class_members:

            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue

            if isinstance(type_instance, DataCollection):
                setattr(attribute_filter, attribute_name, type_instance.get_attribute_filter(default_value))
            else:
                setattr(attribute_filter, attribute_name, default_value)

        return attribute_filter

    def validate(self, value, attribute_filter=None, minified=False):

        if self._required and (value is None or not isinstance(value, dict)):
            """ 
            Model level validation requires a parsed dictionary, this is done by the serializer 
            """
            raise prestans.exception.RequiredAttributeError()
            
        if not value and self._default:
            return self._default
            
        if not self._required and not value:
            """ 
            Value was not provided by caller, but require a template 
            """
            return None
            
        _model_instance = self.__class__()
        _model_class_members = inspect.getmembers(self.__class__)

        rewrite_map = self.attribute_rewrite_map()

        for attribute_name, type_instance in _model_class_members:

            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue

            if attribute_filter and not attribute_filter.is_attribute_visible(attribute_name):
                _model_instance.__dict__[attribute_name] = None
                continue

            if not isinstance(type_instance, DataType):
                raise TypeError("%s must be a DataType subclass" % attribute_name)

            validation_input = None

            input_value_key = attribute_name

            #: Minification support
            if minified is True:
                input_value_key = rewrite_map[attribute_name]
            
            if value.has_key(input_value_key):
                validation_input = value[input_value_key]
                
            try:
                
                if isinstance(type_instance, DataCollection):
                    sub_attribute_filter = None
                    if attribute_filter and attribute_filter.has_key(attribute_name):
                        sub_attribute_filter = getattr(attribute_filter, attribute_name)
                        
                    validated_object = type_instance.validate(validation_input, sub_attribute_filter, minified)
                else:
                    validated_object = type_instance.validate(validation_input)
                
                _model_instance.__dict__[attribute_name] = validated_object

            except prestans.exception.DataValidationException, exp:
                    raise prestans.exception.ValidationError(
                    message=str(exp), 
                    attribute_name=attribute_name, 
                    value=validation_input, 
                    blueprint=type_instance.blueprint())

        return _model_instance

    #: 
    #: Rewrite map generation
    #:

    def attribute_rewrite_map(self):

        rewrite_map = dict()
        model_class_members = inspect.getmembers(self.__class__)

        token_rewrite_map = self._generate_attribute_token_rewrite_map(model_class_members)

        for attribute_name, type_instance in model_class_members:
            
            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue

            if isinstance(type_instance, DataType):
                attribute_tokens = attribute_name.split('_')

                rewritten_attribute_name = ''
                for token in attribute_tokens:
                    rewritten_attribute_name += token_rewrite_map[token] + "_"
                #: Remove the trailing underscore
                rewritten_attribute_name = rewritten_attribute_name[:-1]

                rewrite_map[attribute_name] = rewritten_attribute_name

        return rewrite_map

    def attribute_rewrite_reverse_map(self):

        rewrite_map = dict()
        model_class_members = inspect.getmembers(self.__class__)

        token_rewrite_map = self._generate_attribute_token_rewrite_map(model_class_members)

        for attribute_name, type_instance in model_class_members:
            
            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue

            if isinstance(type_instance, DataType):
                attribute_tokens = attribute_name.split('_')

                rewritten_attribute_name = ''
                for token in attribute_tokens:
                    rewritten_attribute_name += token_rewrite_map[token] + "_"
                #: Remove the trailing underscore
                rewritten_attribute_name = rewritten_attribute_name[:-1]

                rewrite_map[rewritten_attribute_name] = attribute_name

        return rewrite_map

    def has_key(self, attribute_name):

        members = inspect.getmembers(self)
        
        has_key = self.__class__.__dict__.has_key(attribute_name)

        if not has_key:

            base_classes = list()
            base_classes += list(self.__class__.__bases__)

            while not has_key and base_classes:

                #take a base class from the list
                base_class = base_classes.pop()

                #we found it clear the list
                if base_class.__dict__.has_key(attribute_name):
                    has_key = True
                    base_classes = list()
                #add any more base classes from this class
                else:
                    base_classes += list(base_class.__bases__)

        return has_key

    def _generate_attribute_token_rewrite_map(self, model_class_members):

        rewrite_tokens = self._generate_attribute_tokens(model_class_members)
        minified_tokens = self._generate_minfied_keys(len(rewrite_tokens))

        return dict(zip(rewrite_tokens, minified_tokens))

    def _generate_attribute_tokens(self, model_class_members):

        rewrite_tokens = list()

        #: Create a list of tokens
        for attribute_name, type_instance in model_class_members:
            
            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue

            if isinstance(type_instance, DataType):
                rewrite_tokens = rewrite_tokens + attribute_name.split('_')

        #: Remove duplicated; sort alphabetically for the algorithm to work 
        rewrite_tokens = list(set(rewrite_tokens))
        rewrite_tokens.sort()

        return rewrite_tokens

    def _generate_minfied_keys(self, length=26, prefix=''):

        minified_keys = list()

        overflow = 0
        if length > 26:
            overflow = length - 26
            length = 26

        for index in range(0, length):

            generated_char = self._generate_attribute_key(index)
            minified_keys.append(prefix + generated_char)

            if overflow > 0:

                sublist_length = overflow
                if sublist_length > 26:
                    sublist_length = 26

                sublist = self._generate_minfied_keys(sublist_length, generated_char)
                minified_keys = minified_keys + sublist
                overflow = overflow - len(sublist)

        if prefix == '':
            minified_keys.sort(key=len, reverse=False)

        return minified_keys


    def _generate_attribute_key(self, val):
        return string.lowercase[val%26]*(val/26+1)

    #:
    #: Serialization
    #:

    def as_serializable(self, attribute_filter=None, minified=False):
        """
        Returns a dictionary with attributes and pure python representation of 
        the data instances. If an attribute filter is provided as_serializable
        will respect the visibility.

        The response is used by serializers to return data to client
        """
        
        model_dictionary = dict()
        model_class_members = inspect.getmembers(self.__class__)

        rewrite_map = self.attribute_rewrite_map()

        for attribute_name, type_instance in model_class_members:

            serialized_attribute_name = attribute_name

            if attribute_name.startswith('__') or inspect.ismethod(type_instance):
                continue

            if isinstance(attribute_filter, prestans.parser.AttributeFilter) and\
             not attribute_filter.is_attribute_visible(attribute_name):
                continue

            #: Support minification
            if minified is True:
                serialized_attribute_name = rewrite_map[attribute_name]

            if not self.__dict__.has_key(attribute_name) or self.__dict__[attribute_name] is None:
                model_dictionary[serialized_attribute_name] = None
                continue

            if isinstance(type_instance, DataCollection):

                sub_attribute_filter = None
                if isinstance(attribute_filter, prestans.parser.AttributeFilter) and\
                 attribute_filter.has_key(attribute_name):
                    sub_attribute_filter = getattr(attribute_filter, attribute_name)

                model_dictionary[serialized_attribute_name] = self.__dict__[attribute_name].\
                as_serializable(sub_attribute_filter, minified)

            elif isinstance(type_instance, DataStructure):
                python_value = self.__dict__[attribute_name]
                serializable_value = type_instance.as_serializable(python_value)
                model_dictionary[serialized_attribute_name] = serializable_value

            elif isinstance(type_instance, DataType):
                model_dictionary[serialized_attribute_name] = self.__dict__[attribute_name]

        return model_dictionary

#:
#: Body Response Template to transfer binary files
#:

class BinaryResponse(object):

    def __init__(self, mime_type=None, file_name=None, as_attachment=True, contents=None):
        
        if mime_type is not None:
            self._mime_type = mime_type.encode('ascii', 'ignore')
        else:
            self._mime_type = mime_type

        if file_name is not None:
            self._file_name = file_name.encode('ascii', 'ignore')
        else:
            self._file_name = file_name

        self._as_attachment = as_attachment
        self._contents = contents

    @property
    def mime_type(self):
        return self._mime_type

    @mime_type.setter
    def mime_type(self, value):
        self._mime_type = value.encode('ascii', 'ignore')

    @property
    def file_name(self):
        return self._file_name

    @file_name.setter
    def file_name(self, value):
        self._file_name = value.encode('ascii', 'ignore')

    @property
    def as_attachment(self):
        return self._as_attachment

    @as_attachment.setter
    def as_attachment(self, value):
        self._as_attachment = value

    @property
    def contents(self):
        return self._contents

    @contents.setter
    def contents(self, value):
        self._contents = contents

    @property
    def content_length(self):
        if self._contents == None:
            return 0
        return len(self._contents)

    def validate(self):
        return self._mime_type != None\
         and self._file_name != None\
          and self.content_length > 0





# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import re
import webob
import logging
import inspect

import prestans
import prestans.http
import prestans.parser
import prestans.provider
import prestans.exception
import prestans.serializer
import prestans.deserializer

class Request(webob.Request):
    """
    Request is parsed REST Request; it's inherits and relies on Webob.Request to
    do the heavy lifiting of parsing HTTP requests. It adds on top parsing of 
    REST bodies and parameter sets based on rules set by the prestans app.

    It's responsible for making sense of the prestans headers and making them
    available to the RequestHandler
    """

    def __init__(self, environ, charset, logger, deserializers, default_deserializer):

        super(Request, self).__init__(environ=environ, charset=charset)
        self._logger = logger
        self._deserializers = deserializers
        self._default_deserializer = default_deserializer
        self._attribute_filter = None
        self._selected_deserializer = None
        self._parameter_set = None

        self.charset = charset

    @property
    def method(self):
        return self.environ['REQUEST_METHOD']

    @property
    def logger(self):
        return self._logger

    @property
    def parsed_body(self):

        if self.body_template is None:
            raise AttributeError("access to request.parsed_body is not allowed when body_tempalte is set to None")

        return self._parsed_body

    @property
    def supported_mime_types(self):
        return [deserializer.content_type() for deserializer in self._deserializers]

    @property
    def supported_mime_types_str(self):
        return ''.join(str(mime_type) + ',' for mime_type in self.supported_mime_types)[:-1]

    @property
    def selected_deserializer(self):
        return self._selected_deserializer

    @property
    def default_deserializer(self):
        return self._default_deserializer
        
    #: Used by content_type_set to set get a referencey to the serializer object
    def set_deserializer_by_mime_type(self, mime_type):

        for deserializer in self._deserializers:
            if deserializer.content_type() == mime_type:
                self._selected_deserializer = deserializer
                return

        raise prestans.exception.UnsupportedContentTypeError(mime_type, self.supported_mime_types_str)

    @property
    def attribute_filter(self):
        return self._attribute_filter

    @attribute_filter.setter
    def attribute_filter(self, value):
        """
        Attribute filter that used to parse the request_body. This must be set
        before the body_template parameter is set
        """
        self._attribute_filter = value

    @property
    def parameter_set(self):
        return self._parameter_set

    @parameter_set.setter
    def parameter_set(self, value):
        """
        """
        self._parameter_set = value

    @property
    def body_template(self):
        return self._body_template

    @body_template.setter
    def body_template(self, value):
        """
        Must be an instance of a prestans.types.DataCollection subclass; this is
        generally set during the RequestHandler lifecycle. Setting this spwans the
        parsing process of the body. If the HTTP verb is GET an AssertionError is
        thrown. Use with extreme caution.
        """

        if self.method == prestans.http.VERB.GET:
            raise AssertionError("body_template cannot be set for GET requests")

        if value is None:
            self.logger.warn("body_template is None, parsing will be ignored")
            return

        if not isinstance(value, prestans.types.DataCollection):
            raise AssertionError("body_template must be an instance of prestans.types.DataCollection")

        self._body_template = value

        #: Get a deserializer based on the Content-Type header
        #: Do this here so the handler gets a chance to setup extra serializers
        self.set_deserializer_by_mime_type(self.content_type)

        #: Parse the body using the deserializer
        unserialized_body = self.selected_deserializer.loads(self.body)

        #: Parse the body using the remplate and attribute_filter
        self._parsed_body = value.validate(unserialized_body, self.attribute_filter, self.is_minified)

    def register_deserializers(self, deserializers):

        for deserializer in self._deserializers:

            if not isinstance(deserializer, prestans.deserializer.Base):
                raise TypeError("registered deserializer %s.%s does not inherit from prestans.serializer.DeSerializer" % 
                    (deserializer.__module__, deserializer.__class__.__name__))

        self._deserializers = self._deserializers + deserializers


    def get_response_attribute_filter(self, template_filter, template_model=None):
        """
        Prestans-Response-Attribute-List can contain a client's requested 
        definition for attributes required in the response. This should match
        the response_attribute_filter_template? 
        """

        if template_filter is None or not 'Prestans-Response-Attribute-List' in self.headers:
            return None

        #: Header not set results in a None
        attribute_list_str = self.headers['Prestans-Response-Attribute-List']

        #: Deserialize the header contents using the selected header
        json_deserializer = prestans.deserializer.JSON()
        attribute_list_dictionary = json_deserializer.loads(attribute_list_str)

        #: Construct an AttributeFilter
        attribute_filter = prestans.parser.AttributeFilter(from_dictionary=attribute_list_dictionary, 
            template_model=template_model)

        #: Check template? Do this even through we might have template_model
        #: incase users have made a custom filter
        evaluated_filter = attribute_filter.conforms_to_template_filter(template_filter)

        return evaluated_filter

    @property
    def is_minified(self):

        if not 'Prestans-Minification' in self.headers:
            return False

        return self.headers['Prestans-Minification'].upper() == "ON"


class Response(webob.Response):
    """
    Response is the writable HTTP response. It inherits and leverages 
    from webob.Response to do the heavy lifiting of HTTP Responses. It adds to
    weob.Response prestans customisations.

    Overrides content_type property to use prestans' serializers with the set body
    """

    def __init__(self, charset, logger, serializers, default_serializer):
        
        super(Response, self).__init__()

        self._logger = logger
        self._serializers = serializers
        self._default_serializer = default_serializer
        self._selected_serializer = None
        self._template = None
        self._app_iter = []
        self._minify = False
        self._attribute_filter = None
        self._template = None

        #: 
        #: IETF hash dropped the X- prefix for custom headers
        #: http://stackoverflow.com/q/3561381 
        #: http://tools.ietf.org/html/draft-saintandre-xdash-00
        #:
        self.headers.add('Prestans-Version', prestans.__version__)

    @property
    def minify(self):
        return self._minify

    @minify.setter
    def minify(self, value):
        self._minify = value

    @property
    def logger(self):
        return self._logger

    @property
    def supported_mime_types(self):
        return [serializer.content_type() for serializer in self._serializers]

    @property
    def supported_mime_types_str(self):
        return ''.join(str(mime_type) + ',' for mime_type in self.supported_mime_types)[:-1]

    @property
    def selected_serializer(self):
        return self._selected_serializer

    @property
    def default_serializer(self):
        return self._default_serializer

    #: Used by content_type_set to set get a referencey to the serializer object
    def _set_serializer_by_mime_type(self, mime_type):

        for serializer in self._serializers:
            if serializer.content_type() == mime_type:
                self._selected_serializer = serializer
                return

        raise prestans.exception.UnsupportedVocabularyError(mime_type, 
            self.supported_mime_types_str)

    #:
    #: is an instance of prestans.types.DataType; mostly a subclass of 
    #: prestans.types.Model
    #:

    @property
    def template(self):
        return self._template

    @template.setter
    def template(self, value):

        if value is not None and (not isinstance(value, prestans.types.DataCollection) \
            and not isinstance(value, prestans.types.BinaryResponse)):
            raise TypeError("template in response must be of type prestans.types.DataCollection or subclass")

        self._template = value

    #:
    #: Attribute filter rsetup
    #:

    @property
    def attribute_filter(self):
        return self._attribute_filter

    @attribute_filter.setter
    def attribute_filter(self, value):

        if value is not None and not isinstance(value, prestans.parser.AttributeFilter):
            raise TypeError("attribue_filter in response must be of type prestans.types.AttributeFilter")

        self._attribute_filter = value

    #:
    #: content_type; overrides webob.Resposne line 606
    #:

    def _content_type__get(self):
        """
        Get/set the Content-Type header (or None), *without* the
        charset or any parameters.

        If you include parameters (or ``;`` at all) when setting the
        content_type, any existing parameters will be deleted;
        otherwise they will be preserved.
        """
        header = self.headers.get('Content-Type')
        if not header:
            return None
        return header.split(';', 1)[0]

    def _content_type__set(self, value):

        #: Check to see if response can support the requested mime type
        if not isinstance(self._app_iter, prestans.types.BinaryResponse) and value not in self.supported_mime_types:
            raise prestans.exception.UnsupportedVocabularyError(value, self.supported_mime_types_str)

        #: Keep a reference to the selected serializer
        if not isinstance(self._app_iter, prestans.types.BinaryResponse):
            self._set_serializer_by_mime_type(value)

        if not value:
            self._content_type__del()
            return
        if ';' not in value:
            header = self.headers.get('Content-Type', '')
            if ';' in header:
                params = header.split(';', 1)[1]
                value += ';' + params
        self.headers['Content-Type'] = value

    def _content_type__del(self):
        self.headers.pop('Content-Type', None)

    content_type = property(_content_type__get, _content_type__set,
                            _content_type__del, doc=_content_type__get.__doc__)


    #:
    #: body; overrides webob.Response line 324
    #:

    def _body__get(self):
        """
        Overridden response does not support md5, text or json properties. _app_iter
        is set using rules defined by prestans.

        body getter will return the validated prestans model.

        Webob does the heavy lifiting with headers. 
        """

        #: If template is null; return an empty iterable
        if self.template is None:
            return []

        return self._app_iter


    def _body__set(self, value):

        #: If not response template; we have to assume its NO_CONTENT
        #: hence do not allow setting the body
        if self.template is None:
            raise AssertionError("response_template is None; handler can't return a response")

        #: value should be a subclass prestans.types.DataCollection
        if not isinstance(value, prestans.types.DataCollection) and not isinstance(value, prestans.types.BinaryResponse):
            raise TypeError("%s is not a prestans.types.DataCollection or prestans.types.BinaryResponse subclass" % 
                value.__class__.__name__)

        #: Ensure that it matches the return type template
        if not value.__class__ == self.template.__class__:
            raise TypeError("body must of be type %s, given %s" % 
                (self.template.__class__.__name__, value.__class__.__name__))

        #: If it's an array then ensure that element_template matches up
        if isinstance(self.template, prestans.types.Array) and \
        not type(value.element_template) == type(self.template.element_template):
            raise TypeError("array elements must of be type %s, given %s" % 
                (self.template.element_template.__class__.__name__, 
                    value.element_template.__class__.__name__))

        #: _app_iter assigned to value
        #: we need to serialize the contents before we know the length
        #: deffer the content_length property to be set by getter
        self._app_iter = value

    body = property(_body__get, _body__set, _body__set)

    def register_serializers(self, serializers):
        """
        Adds extra serializers; generally registered during the handler lifecycle
        """
        for serializer in serializers:

            if not isinstance(serializer, prestans.serializer.Base):
                raise TypeError("registered serializer %s.%s does not inherit from prestans.serializer.Serializer" % 
                    (serializer.__module__, serializer.__class__.__name__))

        self._serializers = self._serializers + serializers

    def __call__(self, environ, start_response):
        """
        Overridden WSGI application interface
        """

        #: prestans' equivalent of webob.Response line 1022
        if self.template is None or self.status_code == prestans.http.STATUS.NO_CONTENT:

            start_response(self.status, self.headerlist)

            if self.template is not None:
                self.logger.warn("handler returns No Content but has a response_template; set template to None")

            return []

        #: Ensure what we are able to serialize is serializable
        if not isinstance(self._app_iter, prestans.types.DataCollection) and\
         not isinstance(self._app_iter, prestans.types.BinaryResponse):
            raise TypeError("handler returns content of type %s; not a prestans.types.DataCollection subclass" % 
                self._app_iter.__name__)

        if isinstance(self._app_iter, prestans.types.DataCollection):

            #: See if attribute filter is completely invisible
            if self.attribute_filter is not None: 

                #: Warning to say nothing is visible
                if not self.attribute_filter.are_any_attributes_visible():
                    self.logger.warn("attribute_filter has all the attributes turned off, handler will return an empty response")

                #: Warning to say none of the fields match
                model_attribute_filter = None
                if isinstance(self._app_iter, prestans.types.Array):
                    model_attribute_filter = prestans.parser.AttributeFilter.from_model(self._app_iter.element_template)
                elif isinstance(self._app_iter, prestans.types.Model):
                    model_attribute_filter = prestans.parser.AttributeFilter.from_model(self._app_iter)

                if model_attribute_filter is not None:
                    try:
                        model_attribute_filter.conforms_to_template_filter(self.attribute_filter)
                    except prestans.exception.AttributeFilterDiffers, exp:
                        self.logger.warn("%s" %  exp)

            #: Body should be of type DataCollection try; attempt calling
            #: as_seriable with available attribute_filter
            serializable_body = self._app_iter.as_serializable(self.attribute_filter, self.minify)

            #: attempt serializing via registered serializer
            stringified_body = self._selected_serializer.dumps(serializable_body)

            if not type(stringified_body) == str:
                raise TypeError("%s dumps must return a python str not %s" % 
                    (self._selected_serializer.__class__.__name__, 
                        stringified_body.__class__.__name__))

            #: set content_length
            self.content_length = len(stringified_body)

            start_response(self.status, self.headerlist)
            return [stringified_body]

        elif isinstance(self._app_iter, prestans.types.BinaryResponse):

            if self._app_iter.content_length == 0 or self._app_iter.mime_type == None or self._app_iter.file_name == None:
                self.logger.warn("Failed to write binar response with content_length %i; mime_type %s; file_name %s" %
                    (self._app_iter.content_length, self._app_iter.mime_type,self._app_iter.file_name))
                self.status = prestans.http.STATUS.INTERNAL_SERVER_ERROR
                self.content_type = "text/plain"
                return []

            #: Content type
            self.content_type = self._app_iter.mime_type

            #: Add content disposition header
            if self._app_iter.as_attachment:
                self.headers.add('Content-Disposition', "attachment; filename=%s" % self._app_iter.file_name)
            else:                
                self.headers.add('Content-Disposition', "inline; filename=%s" % self._app_iter.file_name)

            #: Write out response
            self.content_length = self._app_iter.content_length

            start_response(self.status, self.headerlist)
            return [self._app_iter.contents]

        else:
            raise AssertionError("prestans failed to write a binary or textual response")

    def __str__(self):
        #: Overridden so webob's __str__ skips serializing the body
        super(Response, self).__str__(skip_body=True)

#:
#: DictionaryResponse serializes dictionaries using the selected_serializer
#:

class DictionaryResponse(Response):

    #:
    #: body; overrides webob.Response line 324
    #:

    def _body__get(self):
        """
        Overridden response does not support md5, text or json properties. _app_iter
        is set using rules defined by prestans.

        body getter will return the validated prestans model.

        Webob does the heavy lifiting with headers. 
        """
        return self._app_iter


    def _body__set(self, value):

        #: value should be a subclass prestans.types.DataCollection
        if not type(value) == dict:
            raise TypeError("%s is not a dictionary" % 
                value.__class__.__name__)

        #: _app_iter assigned to value
        #: we need to serialize the contents before we know the length
        #: deffer the content_length property to be set by getter
        self._app_iter = value

    body = property(_body__get, _body__set, _body__set)

    def __call__(self, environ, start_response):

        start_response(self.status, self.headerlist)

        #: attempt serializing via registered serializer
        stringified_body = self._selected_serializer.dumps(self.body)

        if not type(stringified_body) == str:
            raise TypeError("%s dumps must return a python str not %s" % 
                (self._selected_serializer.__class__.__name__, 
                    stringified_body.__class__.__name__))

        #: set content_length
        self.content_length = len(stringified_body)

        return [stringified_body]

class ErrorResponse(webob.Response):
    """
    ErrorResponse is a specialised webob.Response, its responsible for writing
    out a message in the following format; using the currently selected serializer

      {
          "code": 404,
          "message": "This is an error message",
          "trace": [
            {
                "key": "value"
            }
          ]
      }
    """
    
    def __init__(self, exception, serializer):

        super(ErrorResponse, self).__init__()

        self._exception = exception
        self._serializer = serializer
        self._message = exception.message
        self._stack_trace = exception.stack_trace

        #: 
        #: IETF hash dropped the X- prefix for custom headers
        #: http://stackoverflow.com/q/3561381 
        #: http://tools.ietf.org/html/draft-saintandre-xdash-00
        #:
        self.headers.add('Prestans-Version', prestans.__version__)

        self.content_type = self._serializer.content_type()
        self.status = exception.http_status

    @property
    def trace(self):
        return self._trace

    def append_to_trace(self, trace_entry):
        """
        Use this to append to the stack trace
        """
        self._trace.append(trace_entry)

    def __call__(self, environ, start_response):

        start_response(self.status, self.headerlist)

        error_dict = dict()

        error_dict['code'] = self.status_int
        error_dict['message'] = self._message
        error_dict['trace'] = self._stack_trace

        stringified_body = self._serializer.dumps(error_dict)
        self.content_length = len(stringified_body)

        return [stringified_body]

    def __str__(self):
        #: Overridden so webob's __str__ skips serializing the body
        super(ErrorResponse, self).__str__(skip_body=True)

#:
#: Base Request handler; all handlers must subclass this
#:

class RequestHandler(object):
    """
    RequestHandler is a callable that all API end-points must inherit from. 
    end-points are instantiated by RequestRouter as a match for a URL.

    This class should not be initialised directly. Subclasses should 
    override corresponding methods for HTTP verbs; get, post, delete, put, patch.
    """

    __provider_config__ = prestans.provider.Config()    
    __parser_config__ = prestans.parser.Config()

    def __init__(self, args, request, response, logger, debug):

        self._args = args
        self._request = request
        self._response = response
        self._logger = logger
        self._debug = debug

    @property
    def request(self):
        return self._request

    @property
    def response(self):
        return self._response

    @property
    def logger(self):
        return self._logger

    @property
    def debug(self):
        return self._debug

    def blueprint(self):
        
        handler_blueprint = dict()

        signature_map = [ prestans.http.VERB.GET, \
        prestans.http.VERB.HEAD, \
        prestans.http.VERB.POST, \
        prestans.http.VERB.PUT, \
        prestans.http.VERB.PATCH, \
        prestans.http.VERB.DELETE ]

        #: Provider configuration
        provider_blueprint = None

        if self.__class__.__provider_config__ is not None:
            provider_blueprint = self.__class__.__provider_config__.blueprint()

        handler_blueprint['provider_config'] = provider_blueprint

        # Make a list of methods supported by this handler
        for http_verb in signature_map:

            http_verb_lower = http_verb.lower()
            local_function_handle = getattr(self, http_verb_lower).__func__

            # Ignore if the local signature is the same as the base method
            if local_function_handle is getattr(RequestHandler, http_verb_lower).__func__:
                continue

            verb_blueprint = dict()

            # Docstring
            verb_blueprint['description'] = inspect.getdoc(local_function_handle)
            
            # Arguments, get the first set of parameters for the function handle and ignore echoing self
            verb_blueprint['arguments'] = inspect.getargspec(local_function_handle)[0][1:]

            #: Parser configuration
            parser_blueprint = None

            if self.__class__.__parser_config__ is not None and \
            self.__class__.__parser_config__.get_config_for_verb(http_verb) is not None:

                parser_config = self.__class__.__parser_config__.get_config_for_verb(http_verb)
                parser_blueprint = parser_config.blueprint()

            verb_blueprint['parser_config'] = parser_blueprint

            handler_blueprint[http_verb] = verb_blueprint

        return handler_blueprint

    def _setup_serializers(self):

        #:
        #: Auto set the return serializer based on Accept headers
        #: http://docs.webob.org/en/latest/reference.html#header-getters
        #:

        #: Intersection of requested types and supported types tells us if we
        #: can infact respond in one of the requess formats
        best_accept_match = self.request.accept.best_match(self.response.supported_mime_types,
            default_match=self.response.default_serializer.content_type())

        if best_accept_match is None:
            self.logger.error("unsupported mime type in request; accept header reads %s" % 
                self.request.accept)
            raise prestans.exception.UnsupportedVocabularyError(self.request.accept, 
                self.response.supported_mime_types_str)

        #: If content_type is not acceptable it will raise UnsupportedVocabulary
        self.response.content_type = best_accept_match

    def __call__(self, environ, start_response):

        self.logger.info("handler %s.%s; callable excution start" 
            % (self.__module__, self.__class__.__name__))

        self.logger.info("setting default response to %s" % self.request.accept)

        try:

            #: Register additional serializers and de-serializers
            self.request.register_deserializers(self.register_deserializers())
            self.response.register_serializers(self.register_serializers())

            request_method = self.request.method

            #: Ensure we support the HTTP verb
            if not prestans.http.VERB.is_supported_verb(self.request.method):
                raise prestans.exception.UnimplementedVerbError(self.request.method)

            #: Setup serializers
            self._setup_serializers()

            #: Authentication
            if self.__provider_config__.authentication is not None:
                self.__provider_config__.authentication.debug = self.debug
                self.provider_authentication = self.__provider_config__.authentication

            #: Configuration as provided by the API or default of a VerbConfig object
            verb_parser_config = self.__parser_config__.get_config_for_verb(request_method)

            #: Dress up the request and response with verb configuration
            if verb_parser_config is not None and verb_parser_config.response_template is not None:

                #: Set the response template and attribute filter
                self.response.template = verb_parser_config.response_template

                response_attr_filter_template = verb_parser_config.response_attribute_filter_template

                #: Minification support for response attribute filters
                rewrite_template_model = None
                if self.request.is_minified is True:

                    if isinstance(self.response.template, prestans.types.Array):
                        rewrite_template_model = self.response.template.element_template
                    else:
                        rewrite_template_model = self.response.template

                #: Response attribute filter
                self.response.attribute_filter = self.request.get_response_attribute_filter(response_attr_filter_template, 
                    rewrite_template_model)

                #: If the header is omitted then we ensure the response has a default template
                #: at this point we can assume that we are going to sent down a response
                if self.response.attribute_filter is None:
                    self.response.attribute_filter = response_attr_filter_template

            #: Parameter sets
            if verb_parser_config is not None and len(verb_parser_config.parameter_sets) > 0:

                for parameter_set in verb_parser_config.parameter_sets:

                    if not isinstance(parameter_set, prestans.parser.ParameterSet):
                        raise TypeError("%s not a subclass of ParameterSet" % parameter_set.__class__.__name__)

                    try:
                        validated_parameter_set = parameter_set.validate(self.request)

                        if validated_parameter_set is not None:
                            self.request.parameter_set = validated_parameter_set
                            break

                    except prestans.exception.DataValidationException, exp:
                        self.logger.error(exp)
                        error_response = ErrorResponse(exp, self.response.selected_serializer)
                        return error_response(environ, start_response)

            #: Parse body
            if not request_method == prestans.http.VERB.GET and verb_parser_config is not None:
                self.request.attribute_filter = verb_parser_config.request_attribute_filter
                #: Setting this runs the parser for the body
                #: Request will determine which serializer to use based on Content-Type
                self.request.body_template = verb_parser_config.body_template

            #: Warm up
            self.handler_will_run()

            #:
            #: See if the handler supports the called method
            #: prestans sets a sensible HTTP status code
            #:
            if request_method == prestans.http.VERB.GET:
                self.get(*self._args)
            elif request_method == prestans.http.VERB.HEAD:
                self.head(*self._args)
            elif request_method == prestans.http.VERB.POST:
                self.post(*self._args)
            elif request_method == prestans.http.VERB.PUT:
                self.put(*self._args)
            elif request_method == prestans.http.VERB.PATCH:
                self.patch(*self._args)
            elif request_method == prestans.http.VERB.DELETE:
                self.delete(*self._args)

            #: Tear down
            self.handler_did_run()

            self.logger.info("handler %s.%s; callable excution ends" 
                % (self.__module__, self.__class__.__name__))

            return self.response(environ, start_response)

        except prestans.exception.UnimplementedVerbError, exp:
            self.logger.error(exp)
            error_response = ErrorResponse(exp, self.response.selected_serializer)
            return error_response(environ, start_response)


    #:
    #: Placeholder functions for HTTP Verb; implementing handlers must override these
    #: if not overridden prestans returns a Not Implemented error
    #:

    def register_serializers(self):
        return []

    def register_deserializers(self):
        return []

    def handler_will_run(self):
        return None

    def get(self, *args):
        raise prestans.exception.UnimplementedVerbError(prestans.http.VERB.GET)

    def head(self, *args):
        raise prestans.exception.UnimplementedVerbError(prestans.http.VERB.HEAD)

    def post(self, *args):
        raise prestans.exception.UnimplementedVerbError(prestans.http.VERB.POST)

    def put(self, *args):
        raise prestans.exception.UnimplementedVerbError(prestans.http.VERB.PUT)

    def patch(self, *args):
        raise prestans.exception.UnimplementedVerbError(prestans.http.VERB.PATCH)

    def delete(self, *args):
        raise prestans.exception.UnimplementedVerbError(prestans.http.VERB.DELETE)

    def handler_did_run(self):
        return None

    def redirect(self, url, status=prestans.http.STATUS.TEMPORARY_REDIRECT):

        self._response.status = status
        self._response.headers.add("Location", url)


#:
#: Generates a blueprint of the handler and serializes it using the requested
#: Accept header.
#:

class BlueprintHandler(RequestHandler):

    def __init__(self, args, request, response, logger, debug, route_map):

        super(BlueprintHandler, self).__init__(args, request, response, logger, debug)
        self._route_map = route_map

    def _create_blueprint(self):
        
        blueprint_groups = dict()

        # Intterogate each handler
        for regexp, handler_class in self.route_map:

            # Ignore discovery handler
            if issubclass(handler_class, BlueprintHandler):
                continue

            handler_blueprint = dict()
            handler_blueprint['url'] = regexp
            handler_blueprint['handler_class'] = handler_class.__name__
            handler_blueprint['description'] = inspect.getdoc(handler_class)
            handler_blueprint['supported_methods'] = handler_class(self._args, self.request, 
                self.response, self.logger, self.debug).blueprint()

            # Make a new group per module if one doesnt' exist
            if not handler_class.__module__ in blueprint_groups:
                blueprint_groups[handler_class.__module__] = []

            blueprint_groups[handler_class.__module__].append(handler_blueprint)

        return blueprint_groups

    @property
    def route_map(self):
        return self._route_map

    @route_map.setter
    def route_map(self, value):
        self._route_map = value

    def __call__(self, environ, start_response):

        #: Setup serializers
        self._setup_serializers()

        self.response.status = prestans.http.STATUS.OK

        self.response.body = self._create_blueprint()

        return self.response(environ, start_response)


#:
#: Maps URLs to handlers classes and routes requests based on patterns
#: this is used to init a prestans application
#:

class RequestRouter(object):
    """
    RequestRouter is a specialised WSGI router that primarily maps URLs to Handlers. 
    All registered end-points must inherit from RequestHandler.

    RequestRouter sets the most likely response format based on Accept Headers. If
    no supported response format is found; RequestRouter sends back an HTML error.

    If the requested URL is not handled with the API; RequestRouter presents the
    client with a standardised error message.

    """

    def __init__(self, routes, serializers=None, default_serializer=None, deserializers=None, 
        default_deserializer=None, charset="utf-8", application_name="prestans", 
        logger=None, debug=False, description=None):

        self._application_name = application_name
        self._debug = debug
        self._routes = routes
        self._charset = charset
        self._description = description

        #: Are formats prestans handlers can send data back as
        self._serializers = serializers
        self._default_serializer = default_serializer
        #: Are formats prestans handlers can accept data as
        self._deserializers = deserializers
        self._default_deserializer = default_deserializer

        #:
        #: Init the default logger if one's not provided, this allows users to configure their own
        #: http://www.blog.pythonlibrary.org/2012/08/02/python-101-an-intro-to-logging/
        #:
        if logger is None:
            logging.basicConfig()
            self._logger = logging.getLogger("prestans.%s" % application_name)
        else:
            self._logger = logger

        #: Set logger level, API can override this
        if self._debug == True:
            self._logger.setLevel(logging.DEBUG)
        else:
            self._logger.setLevel(logging.ERROR)

        #: If serializers and deserialers aren't provided, prestans runs as a JSON app
        if serializers is None:
            self._serializers = [prestans.serializer.JSON()]

        if default_serializer is None:
            self._default_serializer = prestans.serializer.JSON()

        #: Deserializers
        
        if deserializers is None:
            self._deserializers = [prestans.deserializer.JSON()]

        if default_deserializer is None:
            self._default_deserializer = prestans.deserializer.JSON()

    @property
    def logger(self):
        return self._logger

    @property
    def description(self):
        return self._description

    @property
    def charset(self):
        return self._charset

    @property
    def debug(self):
        return self._debug

    @property
    def application_name(self):
        return self._application_name

    def __call__(self, environ, start_response):

        #: Say hello
        self._logger.info("%s exposes %i end-points; prestans %s; charset %s; debug %s" % (
            self._application_name, len(self._routes), prestans.__version__, self._charset, self._debug))

        #: Validate serailziers and deserialzers; are subclasses of prestans.serializer.Serializer
        _default_outgoing_mime_types = list()
        for serializer in self._serializers:

            if not isinstance(serializer, prestans.serializer.Base):
                raise TypeError("registered serializer %s.%s does not inherit from prestans.serializer.Serializer" % 
                    (serializer.__module__, serializer.__class__.__name__))

            _default_outgoing_mime_types.append(serializer.content_type())

        _default_incoming_mime_types = list()
        for deserializer in self._deserializers:

            if not isinstance(deserializer, prestans.deserializer.Base):
                raise TypeError("registered deserializer %s.%s does not inherit from prestans.serializer.DeSerializer" % 
                    (deserializer.__module__, deserializer.__class__.__name__))

            _default_incoming_mime_types.append(deserializer.content_type())

        #: Report on the acceptable mime types
        self._logger.info("generally accepts %s; speaks %s" % 
            (str(_default_outgoing_mime_types).strip("[]'"), str(_default_incoming_mime_types).strip("[]'")))

        #: Attempt to parse the HTTP request
        request = Request(environ=environ, charset=self._charset, logger=self._logger, 
            deserializers=self._deserializers, default_deserializer=self._default_deserializer)

        #: Initialise the Route map
        route_map = self._init_route_map(self._routes)

        try:

            #: Check if the requested URL has a valid registered handler
            for regexp, handler_class in route_map:

                match = regexp.match(request.path)

                #: If we've found a match; ensure its a handler subclass and return it's callable
                if match:

                    if issubclass(handler_class, BlueprintHandler):

                        response = DictionaryResponse(charset=self._charset, logger=self._logger, 
                            serializers=self._serializers, default_serializer=self._default_deserializer)

                        request_handler = handler_class(args=match.groups(), request=request, response=response, 
                            logger=self._logger, debug=self._debug, route_map=self._routes)

                    else:

                        response = Response(charset=self._charset, logger=self._logger, serializers=self._serializers, 
                            default_serializer=self._default_deserializer)

                        response.minify = request.is_minified

                        request_handler = handler_class(args=match.groups(), request=request, response=response, 
                            logger=self._logger, debug=self._debug)

                    return request_handler(environ, start_response)

            #: Request does not have a matched handler
            raise prestans.exception.NoEndpointError()

        except prestans.exception.Base, exp:
            self.logger.error(exp)
            error_response = ErrorResponse(exp, self._default_serializer)
            return error_response(environ, start_response)

    def _init_route_map(self, routes):

        parsed_handler_map = []
        handler_name = None
        
        for regexp, handler in routes:

            try:
                handler_name = handler.__name__
            except AttributeError:
                pass

            #: Patch regular expression if its incomplete 
            if not regexp.startswith('^'):
                regexp = '^' + regexp
            if not regexp.endswith('$'):
                regexp += '$'
                
            compiled_regex = re.compile(regexp)
            parsed_handler_map.append((compiled_regex, handler))
            
        return parsed_handler_map



# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import inspect

class ModelAdapter(object):
    
    def __init__(self, rest_model_class, persistent_model_class):
        self._rest_model_class = rest_model_class
        self._persistent_model_class = persistent_model_class

    @property
    def persistent_model_class(self):
        return self._persistent_model_class
    
    @property
    def rest_model_class(self):
        return self._rest_model_class
            
    def adapt_persistent_to_rest(self, persistent_object):
        raise AssertionError("adapt_persistent_to_rest direct use not allowed")

class AdapterRegistryManager:
    """
    AdapterRegistryManager keeps track of rest to persistent model maps

    AdapterRegistryManager should not be instantiated by the applications, a singleton
    instance supplied by this package.
    """

    def __init__(self):
        self._persistent_map = dict()
        self._rest_map = dict()

    def register_adapter(self, model_adapter):
        
        if not isinstance(model_adapter, ModelAdapter):
            raise TypeError("Registry recd instance of type %s is not a ModelAdapter" 
                % model_adapter.__class__.__name__)
        
        rest_class_signature = model_adapter.rest_model_class.__module__ + "." + model_adapter.rest_model_class.__name__
        persistent_class_signature = model_adapter.persistent_model_class.__module__ + "." + model_adapter.persistent_model_class.__name__
        
        #:
        #: Store references to how a rest model maps to a persistent model and vice versa 
        #:
        self._persistent_map[persistent_class_signature] = model_adapter
        self._rest_map[rest_class_signature] = model_adapter
        
    def get_adapter_for_persistent_model(self, persistent_model):
        
        class_signature = persistent_model.__class__.__module__ + "." + persistent_model.__class__.__name__
        
        if not self._persistent_map.has_key(class_signature):
            raise TypeError("No registered Data Adapter for class %s" % class_signature)

        return self._persistent_map[class_signature]
        
    def get_adapter_for_rest_model(self, rest_model):
        
        class_signature = rest_model.__class__.__module__ + "." + rest_model.__class__.__name__
        
        if not self._rest_map.has_key(class_signature):
            raise TypeError("No registered Data Adapter for class %s" % class_signature)

        return self._rest_map[class_signature]


#:
#: Singleton instantiated if adapeter package is imported
#:
registry = AdapterRegistryManager()



# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
__all__ = ['closure', 'templates']

import inspect
import jinja2
import os
import sys

import prestans.devel.gen.closure
import prestans.types

class Inspector(object):

    def __init__(self, model_file):
        self._model_file = model_file

    def inspect(self):

        blueprints = list()

        try:
            sys.path.append(os.path.dirname(self._model_file))
            model_python_file = os.path.split(self._model_file)[1:][0]
            namespace = model_python_file.split(".")[0]
            models = __import__(namespace, globals(), locals(), [])
        except ImportError, import_error:
            print "\nFailed to process %s, %s" % (self._model_file, import_error)
            sys.exit(1)

        for name, type_instance in models.__dict__.iteritems():
            if name.startswith('__') or inspect.ismethod(type_instance) or not inspect.isclass(type_instance) or\
             not issubclass(type_instance, prestans.types.Model):
                continue

            blueprint = type_instance().blueprint()
            for field_name, field_blueprint in blueprint['fields'].iteritems():
                field_blueprint['map_name'] = type_instance().attribute_rewrite_map()[field_name]
            blueprints.append(blueprint)


        return blueprints

class Preplate(object):

    def __init__(self, template_type, model_file, namespace, filter_namespace, output_directory):
        
        self._template_type = template_type
        self._model_file = model_file
        self._namespace = namespace
        self._filter_namespace = filter_namespace
        self._output_directory = output_directory
        loader = jinja2.PackageLoader('prestans', 'devel/gen/templates')
        self._template_engine = jinja2.Environment(trim_blocks=True, loader=loader)
        self._template_engine.globals["isinstance"] = isinstance
        self._template_engine.globals["list"] = list

    def run(self):

        template = None
        if self._template_type == "closure.model":
            template = prestans.devel.gen.closure.Model(template_engine=self._template_engine, 
                model_file=self._model_file, namespace=self._namespace, filter_namespace=self._filter_namespace, output_directory=self._output_directory)
        elif self._template_type == "closure.filter":
            template = prestans.devel.gen.closure.Filter(template_engine=self._template_engine, 
                model_file=self._model_file, namespace=self._namespace, output_directory=self._output_directory)

        if template is None:
            return 1
        
        return template.run()


# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import unittest

import prestans.types
import prestans.exception

class StringTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._string = prestans.types.String(
            default="orange",
            min_length=1,
            max_length=20,
            choices=['orange','mango','banana'],
            )

    def test_default(self):
        self.assertEqual(self._string._default, "orange", "Check default value")

    def test_length(self):
        pass

    def test_regex_format(self):
        pass

    def test_choices(self):
        pass

    def test_encoding(self):
        pass

    def tearDown(self):
        self._string = None

class IntegerTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._integer = prestans.types.Integer(
            default=1,
            minimum=1,
            maximum=20,
            )

    def test_default(self):
        self.assertEqual(self._integer.validate(1), 1)

    def test_minimum(self):
        self.assertRaises(prestans.exception.LessThanMinimumError, self._integer.validate(-1))

    def text_maximum(self):
        self.assertRaises(prestans.exception.MoreThanMaximumError, self._integer.validate(21))

    def test_choices(self):
        self.assertRaises(prestans.exception.InvalidChoiceError, self._integer.validate(2))

    def tearDown(self):
        self._integer = None


class FloatTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._float = prestans.types.Float(
            default=1.0,
            minimum=1.0,
            maximum=20.0,
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        self._float = None

class BooleanTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._boolean = prestans.types.Boolean(
            default=True,
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        self._boolean = None

class DataURLFileTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._data_url_file = prestans.types.DataURLFile(
            allowed_mime_types=[],
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        self._data_url_file = None


class DateTimeTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._datetime = prestans.types.DateTime(
            default=True,
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        self._datetime = None

class DateTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._date = prestans.types.Date(
            default=True,
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        self._date = None

class TimeTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._time = prestans.types.Time(
            default=True,
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        self._date = None

class ArrayTypeUnitTest(unittest.TestCase):

    def setUp(self):        
        self._string_array = prestans.types.Array(
            element_template=prestans.types.String()
        )
        self._integer_array = prestans.types.Array(
            element_template=prestans.types.Integer()
        )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def test_append(self):
        pass

    def tearDown(self):
        self._array = None

class ModelTypeUnitTest(unittest.TestCase):

    class TestModel(prestans.types.Model):

        string = prestans.types.String(required=True, max_length=10)

    def setUp(self):
        
        self._model = ModelTypeUnitTest.TestModel()

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        self._model = None

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__all__ = ['DeSerializer', 'JSON', 'XMLPlist']

import prestans.exception

class Base(object):

    def loads(self, input_string):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)

    def content_type(self):
        raise TypeError("%s should not be used directly" % self.__class__.__name__)


class JSON(Base):

    def loads(self, input_string):

        import json
        parsed_json = None

        try:
            parsed_json = json.loads(input_string)
        except Exception, exp:
            raise prestans.exception.DeSerializationFailedError('JSON')
            
        return parsed_json
        
    def content_type(self):
        return 'application/json'


class XMLPlist(Base):

    def loads(self, input_string):
        
        import plistlib
        unserialized_plist = None

        try:
            unserialized_plist = plistlib.readPlistFromString(input_string)
        except Exception, exp:
            raise prestans.exception.DeSerializationFailedError("XML/Plist")

        return unserialized_plist

    def content_type(self):
        return 'application/xml'
        


# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__all__ = ['ModelAdapter']

#:
#: prestans.ext.data.adapters.sqlalchemy SQLAlchemy specific implementation of Data Adapter
#:

import inspect

import prestans.types
import prestans.parser

from prestans.ext.data import adapters

#:
#: Adapt an instance
#:
def adapt_persistent_instance(persistent_object, target_rest_class=None, attribute_filter=None):
    """
    Adapts a single persistent instance to a REST model; at present this is a
    common method for all persistent backends.

    This might be moved to backend specific packages if the need arrises

    Refer to: https://groups.google.com/forum/#!topic/prestans-discuss/dO1yx8f60as
    for discussion on this feature
    """

    #: Try and get the adapter and the REST class for the persistent object 
    if target_rest_class is None:
        adapter_instance = adapters.registry.get_adapter_for_persistent_model(collection[0])
    else:
        if inspect.isclass(target_rest_class):
            target_rest_class = target_rest_class()
        
        adapter_instance = adapters.registry.get_adapter_for_rest_model(target_rest_class)

    #: Would raise an exception if the attribute_filter differs from the target_rest_class
    if attribute_filter is not None and isinstance(attribute_filter, prestans.parser.AttributeFilter):
        prestans.parser.AttributeFilter.from_model(target_rest_class).conforms_to_template_filter(attribute_filter)

    return adapter_instance.adapt_persistent_to_rest(persistent_object, attribute_filter)

#:
#: Adapt an instance
#:
def adapt_persistent_collection(persistent_collection, target_rest_class=None, attribute_filter=None):
        
    #: Ensure that colleciton is iterable and has atleast one element 
    persistent_collection_length = 0
    
    #: Attempt to reliably detect the length of the persistent_collection 
    if persistent_collection and isinstance(persistent_collection, (list, tuple)):
        persistent_collection_length = len(persistent_collection)
    elif persistent_collection and persistent_collection.__module__ == "sqlalchemy.orm.query":
        persistent_collection_length = persistent_collection.count()
        
    #: If the persistent_collection is empty then return a blank array 
    if persistent_collection_length == 0:
        return prestans.types.Array(element_template=target_rest_class())
        
    #: Try and get the adapter and the REST class for the persistent object 
    if target_rest_class is None:
        adapter_instance = adapters.registry.get_adapter_for_persistent_model(persistent_collection[0])
    else:
        if inspect.isclass(target_rest_class):
            target_rest_class = target_rest_class()
        
        adapter_instance = adapters.registry.get_adapter_for_rest_model(target_rest_class)
        
    adapted_models = prestans.types.Array(element_template=adapter_instance.rest_model_class())
    
    for persistent_object in persistent_collection:
        adapted_models.append(adapter_instance.adapt_persistent_to_rest(persistent_object, attribute_filter))
        
    return adapted_models
    
class ModelAdapter(adapters.ModelAdapter):
    """
    Provide a brige between REST models and SQLAlchemy objects
    """
    
    def adapt_persistent_to_rest(self, persistent_object, attribute_filter=None):
        """
        adapts a persistent model to a rest model by inspecting
        """

        rest_model_instance = self.rest_model_class()

        for attribute_key in rest_model_instance.get_attribute_keys():                           

            rest_attr = getattr(self.rest_model_class, attribute_key)

            if not hasattr(persistent_object, attribute_key):
                # Don't bother processing if the persistent model doesn't have this attribute

                if isinstance(rest_attr, prestans.types.Model):
                    #: If the attribute is a Model, then we set it to None otherwise we get a model 
                    #: with default values, which is invalid when constructing responses
                    try:
                        setattr(rest_model_instance, attribute_key, None)
                    #: Catch any exception thrown from setattr to give a useable error message
                    except TypeError, exp:
                        raise TypeError('Attribute %s, %s' % (attribute_key, str(exp)))

                continue

            #: Attribute not visible don't bother processing
            elif isinstance(attribute_filter, prestans.parser.AttributeFilter) and\
             not attribute_filter.is_attribute_visible(attribute_key):
                continue

            elif isinstance(rest_attr, prestans.types.Array):        
                #: Handles prestans array population from SQLAlchemy relationships 

                persistent_attr_value = getattr(persistent_object, attribute_key)
                rest_model_array_handle = getattr(rest_model_instance, attribute_key)
                
                #: Iterator uses the .append method exposed by prestans arrays to validate
                #: and populate the collection in the instance.
                for collection_element in persistent_attr_value:
                    if isinstance(rest_attr.element_template, prestans.types.String):
                        rest_model_array_handle.append(collection_element)
                    elif isinstance(rest_attr.element_template, prestans.types.Integer):
                        rest_model_array_handle.append(collection_element)
                    elif isinstance(rest_attr.element_template, prestans.types.Float):
                        rest_model_array_handle.append(collection_element)
                    elif isinstance(rest_attr.element_template, prestans.types.Boolean):
                        rest_model_array_handle.append(collection_element)
                    else:
                        element_adapter = adapters.registry.get_adapter_for_rest_model(rest_attr.element_template)

                        #: Check if there is a sub model filter
                        sub_attribute_filter = None
                        if attribute_filter and attribute_filter.has_key(attribute_key):
                            sub_attribute_filter = getattr(attribute_filter, attribute_key)

                        adapted_rest_model = element_adapter.adapt_persistent_to_rest(collection_element, sub_attribute_filter)                    
                        rest_model_array_handle.append(adapted_rest_model)
            
            elif isinstance(rest_attr, prestans.types.Model):
                
                try:
                    persistent_attr_value = getattr(persistent_object, attribute_key)
                    
                    if persistent_attr_value is None:
                        adapted_rest_model = None
                    else:
                        model_adapter = adapters.registry.get_adapter_for_rest_model(rest_attr)

                        #: Check if there is a sub model filter
                        sub_attribute_filter = None
                        if isinstance(attribute_filter, prestans.parser.AttributeFilter) and attribute_filter.has_key(attribute_key):
                            sub_attribute_filter = getattr(attribute_filter, attribute_key)

                        adapted_rest_model = model_adapter.adapt_persistent_to_rest(persistent_attr_value, sub_attribute_filter)

                    setattr(rest_model_instance, attribute_key, adapted_rest_model)
                    
                except TypeError, exp:
                    raise TypeError('Attribute %s, %s' % (attribute_key, str(exp)))
                except prestans.exception.DataValidationException, exp:
                    raise prestans.exception.InconsistentPersistentDataError(attribute_key, str(exp))
                
            else:
                
                #: Otherwise copy the value to the rest model 
                try:
                    persistent_attr_value = getattr(persistent_object, attribute_key)
                    setattr(rest_model_instance, attribute_key, persistent_attr_value)
                except TypeError, exp:
                    raise TypeError('Attribute %s, %s' % (attribute_key, str(exp)))
                except prestans.exception.DataValidationException, exp:
                    raise prestans.exception.InconsistentPersistentDataError(attribute_key, str(exp))

        return rest_model_instance
        


# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#



# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__all__ = [
    'UnsupportedVocabularyError',
    'UnsupportedContentTypeError',
    'ValidationError',
    'DataValidationException',
    'ParserException',
    'HandlerException',

    #: Parser Exceptions
    'NoEndpointError',
    'SerializationFailed',

    #: Data Validation
    'RequiredAttributeError',
    'ParseFailedError',
    'InvalidValue',
    'LessThanMinimum',
    'MoreThanMaximum',
    'InvalidChoice',
    'UnacceptableLength',
    'InvalidType',
    'InvalidCollection',
    'MissingParameter',
    'InvalidFormat',
    'InvalidMetaValue',
    'UnregisteredAdapter',

    #: Handler exceptions
    'ServiceUnavailable',
    'BadRequest',
    'Conflict',
    'NotFound',
    'Unauthorized',
    'MovedPermanently',
    'PaymentRequired',
    'Forbidden'
]

import prestans
import prestans.http

class Base(Exception):

    def __init__(self, http_status, message):

        self._http_status = http_status
        self._message = message
        self._stack_trace = list()

    @property
    def http_status(self):
        return self._http_status

    @http_status.setter
    def http_status(self, value):
        self._http_status = value

    @property
    def stack_trace(self):
        return self._stack_trace

    def push_trace(self, trace_object):
        self._stack_trace.append(trace_object)

    @property
    def message(self):
        return self._message

    def __unicode__(self):
        return unicode(self._message)

    def __str__(self):
        return unicode(self._message).encode('utf-8')

#:
#: These are top level exceptions that layout tell prestans how
#: the resulting messages are written out to the either the client
#: or the logger
#:
#: None of these exceptions are used directly, look at the sections
#: for usable exceptions.
#:
#: According to PEP 008 all exceptions that are error should have
#: the Error suffix e.g UnsupportedVocabularyError
#: http://www.python.org/dev/peps/pep-0008/#exception-names
#:

class UnsupportedVocabularyError(Base):
    """
    Called if none of the requested vocabularies are supported by the API
    this error message is sent as an HTML document. This exception is the
    only one that breaks serialisation rules.
    """

    def __init__(self, accept_header, supported_types):

        _code = prestans.http.STATUS.NOT_IMPLEMENTED
        _message = "Unsupported vocabulary in the Accept header"
        super(UnsupportedVocabularyError, self).__init__(_code, _message)

        self.push_trace({ 
            'accept_header': str(accept_header),
            'supported_types': supported_types
            })
    
class UnsupportedContentTypeError(Base):
    
    def __init__(self, requested_mime_type, content_type):

        _code = prestans.http.STATUS.NOT_IMPLEMENTED
        _message = "Unsupported Content-Type in Request"
        super(UnsupportedContentTypeError, self).__init__(_code, _message)

        self.push_trace({ 
            'requested_type': requested_mime_type,
            'supported_types': content_type
            })

class ValidationError(Base):
    """
    DataValidationException are Exceptions raised if prestans fails
    to validate data inbound or out using rules defined in Models.

    These contains a stack trace to indicate the depth of the error.
    E.g Model has Sub Model that has attribute which failed to validate.

    Each exception uses an HTTP status code and is sent to the client.
    """
    def __init__(self, message, attribute_name, value, blueprint):

        super(ValidationError, self).__init__(prestans.http.STATUS.BAD_REQUEST, message)
        self._attribute_name = attribute_name
        self._value = str(value)

        self.append_validation_trace(blueprint)

    def append_validation_trace(self, blueprint):
        self.push_trace({
            'attribute_name': self._attribute_name,
            'value': self._value,
            'message': self._message,
            'blueprint': blueprint
            })

    #:
    #: The following are overridden to ensure sensible logging messages
    #:

    def __unicode__(self):
        _loggable_message = "%s %s" % (self._attribute_name, self._message)
        return unicode(_loggable_message)

    def __str__(self):
        _loggable_message = "%s %s" % (self._attribute_name, self._message)
        return unicode(_loggable_message).encode('utf-8')

class ParserException(Base):
    """
    ParserException are Exceptions raised if prestans fails to parse
    a request; these generally revolve around the Content-Types or 
    missing payload. Specific parsing messages are of type DataValidationException
    """
    def __init__(self, code, message):
        super(ParserException, self).__init__(code, message)

class HandlerException(Base):
    """
    HandlerExceptions are Exceptions that are raised by handlers of the
    REST application. REST encourages the use different error codes to
    denote what the error is.

    E.g Use 404 to denote that a reqeusted doesn't exists on the server.

    If one of these exceptions do not match your user case; you are free
    construct your own error message and use an error code from outlined
    in prestans.http
    """
    def __init__(self, code, message):
        super(HandlerException, self).__init__(code, message)

#:
#: Parser Exception
#:

class UnimplementedVerbError(ParserException):

    def __init__(self, verb_name):

        _code = prestans.http.STATUS.NOT_IMPLEMENTED
        _message = "API does not implement the HTTP Verb"
        super(UnimplementedVerbError, self).__init__(_code, _message)

        self.push_trace({ 
            'verb': verb_name,
            })

class NoEndpointError(ParserException):

    def __init__(self):

        _code = prestans.http.STATUS.NOT_FOUND
        _message = "API does not provide this end-point"
        super(NoEndpointError, self).__init__(_code, _message)

class AuthenticationError(ParserException):

    def __init__(self, message=None):

        _code = prestans.http.STATUS.UNAUTHORIZED
    
        _message = message
        if _message is None:
            _message = "Authentication Error; service is only available to authenticated"
    
        super(AuthenticationError, self).__init__(_code, _message)

class AuthorizationError(ParserException):

    def __init__(self, role_name):

        _code = prestans.http.STATUS.FORBIDDEN
        _message = "%s is not allowed to access this resource"
        super(AuthorizationError, self).__init__(_code, _message)


class SerializationFailedError(ParserException):

    def __init__(self, format):

        _code = prestans.http.STATUS.NOT_FOUND
        _message = "Serialization failed: %s" % format
        super(SerializationFailedError, self).__init__(_code, _message)

class DeSerializationFailedError(ParserException):
    
    def __init__(self, format):

        _code = prestans.http.STATUS.NOT_FOUND
        _message = "DeSerialization failed: %s" % format
        super(DeSerializationFailedError, self).__init__(_code, _message)

class AttributeFilterDiffers(ParserException):
    """
    AttributeFilter initialised from request input does not conform to
    the configured template.
    """

    def __init__(self, attribute_list):

        _code = prestans.http.STATUS.BAD_REQUEST
        _message = "attribute filter does not contain attributes (%s) that are not part of template" % (', '.join(attribute_list))
        super(AttributeFilterDiffers, self).__init__(_code, _message)

        self.push_trace({ 
            'rejected_attribute_list': attribute_list,
            })

class InconsistentPersistentDataError(Base):
    """
    Raised by Data Adapters if validation of stored data fails. Extra information is
    written out to the server log and the client is returned a 500 response.

    Practical expectation is that this should not occur; however if REST rules have 
    changed since the instantiation of the system.
    """

    def __init__(self, attribute_name, exception_message):
        _code = prestans.http.STATUS.INTERNAL_SERVER_ERROR
        _message = "Data Adapter failed to validate stored data on the server"
        super(InconsistentPersistentDataError, self).__init__(_code, _message)

        self._attribute_name = attribute_name

        self.push_trace({
            'exception_message': exception_message,
            'attribute_name': attribute_name
            })

    def __str__(self):
        return "DataAdapter failed to adapt %s, %s" % (self._attribute_name, self.message)

#:
#: Data Validation
#: 

class DataValidationException(Base):

    def __init__(self, message):
        super(DataValidationException, self).__init__(prestans.http.STATUS.BAD_REQUEST, message)

class RequiredAttributeError(DataValidationException):

    def __init__(self, attribute_name=None):
        _message = "attribute is required and does not provide a default value"
        super(RequiredAttributeError, self).__init__(_message)

class ParseFailedError(DataValidationException):
    
    def __init__(self, message="Parser Failed"):
        _message = message
        super(ParseFailedError, self).__init__(_message)

class LessThanMinimumError(DataValidationException):
    
    def __init__(self, value, allowed_min):
        _message = "%i is less than the allowed minimum of %i" % (value, allowed_min)
        super(LessThanMinimumError, self).__init__(_message)

class MoreThanMaximumError(DataValidationException):

    def __init__(self, value, allowed_max):
        _message = "%i is more than the allowed maximum %i" % (value, allowed_max)
        super(MoreThanMaximumError, self).__init__(_message)

class InvalidChoiceError(DataValidationException):

    def __init__(self, value, allowed_choices):
        _message = "value is not one of these choices %s" % str(allowed_choices).strip('[]')
        super(InvalidChoiceError, self).__init__(_message)

class UnacceptableLengthError(DataValidationException):
    
    def __init__(self, value, minimum, maximum):
        _message = "value has to be %i and %i" % (minimum, maximum)
        super(UnacceptableLengthError, self).__init__(_message)

class InvalidType(DataValidationException):
    
    def __init__(self, value, type_name):
        _message = "invalid data type %s" % type_name
        super(InvalidType, self).__init__(_message)

class MissingParameterError(DataValidationException):

    def __init__(self):
        _message = "missing parameter"
        super(MissingParameterError, self).__init__(_message)

class InvalidFormatError(DataValidationException):
    
    def __init__(self, value):
        _message = "invalid value %s provided" % value
        super(InvalidFormatError, self).__init__(_message)

class InvalidMetaValueError(DataValidationException):

    def __init__(self):
        _message = "invalid meta value"
        super(InvalidMetaValueError, self).__init__(_message)

class UnregisteredAdapterError(DataValidationException):

    def __init__(self):
        _message = "no registered adapters for data model"
        super(UnregisteredAdapterError, self).__init__(_message)

#:
#: The following excepetions are used by REST handlers to indicate 
#: commonly defined scenarios when dealing with data. BaseHandler traps 
#: these and generates an appropriate error message for the end user.
#:

class ServiceUnavailable(HandlerException):
    
    def __init__(self, message):
        _code = prestans.http.STATUS.SERVICE_UNAVAILABLE
        super(ServiceUnavailable, self).__init__(_code, message)

class BadRequest(HandlerException):

    def __init__(self, message):
        _code = prestans.http.STATUS.BAD_REQUEST
        super(BadRequest, self).__init__(_code, message)

class Conflict(HandlerException):

    def __init__(self, message):
        _code = prestans.http.STATUS.CONFLICT
        super(Conflict, self).__init__(_code, message)

class NotFound(HandlerException):

    def __init__(self, message):
        _code = prestans.http.STATUS.NOT_FOUND
        super(NotFound, self).__init__(_code, message)

class Unauthorized(HandlerException):

    def __init__(self, message):
        _code = prestans.http.STATUS.UNAUTHORIZED
        super(Unauthorized, self).__init__(_code, message)

class MovedPermanently(HandlerException):

    def __init__(self, message):
        _code = prestans.http.STATUS.MOVED_PERMANENTLY
        super(MovedPermanently, self).__init__(_code, message)

class PaymentRequired(HandlerException):

    def __init__(self, message):
        _code = prestans.http.STATUS.PAYMENT_REQUIRED
        super(PaymentRequired, self).__init__(_code, message)

class Forbidden(HandlerException):

    def __init__(self, message):
        _code = prestans.http.STATUS.FORBIDDEN
        super(Forbidden, self).__init__(_code, message)



# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2015, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


__all__ = ['gen', 'serve']

import argparse
import os

#import prestans.devel.serve
#import prestans.devel.gen
import exception

class ArgParserFactory(object):

    #:
    #:
    #:
    def __init__(self):

        self._arg_parser = argparse.ArgumentParser(
            description="command line tools to compliment the prestans framework",
            epilog="pride is distributed by the prestans project <http://github.com/anomaly/prestans/> under the the New BSD license."
        )

        subparsers_handle = self._arg_parser.add_subparsers(dest="sub_command")

        self._add_generate_build_commands(subparsers_handle)
        self._add_generate_sub_commands(subparsers_handle)
        self._add_server_sub_commands(subparsers_handle)

    #:
    #: public message to fire argparser
    #:
    def parse(self):
        return self._arg_parser.parse_args()

    #:
    #: build subcommand
    #:
    def _add_generate_build_commands(self, subparsers_handle):

        gen_parser = subparsers_handle.add_parser(
            name="build",
            help="builds deployable javascript, css and server"
            )

        build_sub_parser = gen_parser.add_subparsers(dest="build-sub-commands")

        #:
        #: build a distribution
        #:
        build_sub_parser.add_parser(
            name="dist",
            help="builds distributable application"
            )

        #:
        #: Javascript
        #:
        build_sub_parser.add_parser(
            name="js",
            help="builds distributable javascript"
            )

        #:
        #: CSS
        #:
        build_sub_parser.add_parser(
            name="css",
            help="builds distributable css"
            )

    #:
    #: gen subcommand
    #:
    def _add_generate_sub_commands(self, subparsers_handle):

        gen_parser = subparsers_handle.add_parser(
            name="gen",
            help="generate client side model stubs, filters"
            )
        
        gen_parser.add_argument(
            "-t",
            "--template",
            choices=['closure.model', 'closure.filter'],
            default='closure.model',
            required=True,
            dest="template",
            help="template to use for client side code generation"
            )

        gen_parser.add_argument(
            "-m",
            "--model",
            required=True,
            dest="model_path",
            help="path to models description file"
            )


        gen_parser.add_argument(
            "-o",
            "--output",
            default=".",
            dest="output",
            help="output path for generated code"
            )

        gen_parser.add_argument(
            "-n",
            "--namespace",
            required=True,
            dest="namespace",
            help="namespace to use with template e.g prestans.data.model"
            )

        gen_parser.add_argument(
            "-fn",
            "--filter-namespace",
            required=False,
            default=None,
            dest="filter_namespace",
            help="filter namespace to use with template e.g prestans.data.filter"
            )



    #:
    #: server subcommand
    #:
    #: --config path to configuration file
    #:
    def _add_server_sub_commands(self, subparsers_handle):

        server_parser = subparsers_handle.add_parser(
            name="serve",
            help="runs a local HTTP WSGI server for your prestans project"
            )

        server_parser.add_argument(
            "-c",
            "--config",
            default="./devserver.yaml",
            dest="config_path",
            help="path to prestans devserver configuration"
            )


class CommandDispatcher:

    def __init__(self, args):
        self._args = args

    def dispatch(self):
        
        if self._args.sub_command == "gen":
            self._dispatch_gen()
        elif self._args.sub_command == "build":
            self._dispatch_build()
        elif self._args.sub_command == "serve":
            self._dispatch_serve()

    def _dispatch_gen(self):

        if not os.path.isdir(self._args.output):
            raise exception.Base("%s is not a writeable directory" % self._args.output)

        if not os.path.isfile(self._args.model_path):
            raise exception.Base("failed to read model file %s" % self._args.model_path)

        import prestans.devel.gen
        preplate = prestans.devel.gen.Preplate(
            template_type=self._args.template, 
            model_file=self._args.model_path, 
            namespace=self._args.namespace,
            filter_namespace=self._args.filter_namespace,
            output_directory=self._args.output)

        preplate.run()

    def _dispatch_build(self):
        print self._args

    def _dispatch_serve(self):
        import prestans.devel.serve
        server_config = prestans.devel.serve.Configuration(self._args.config_path)
        dev_server = serve.DevServer(server_config)
        dev_server.run()
        